Package: tricycle
Type: Package
Title: tricycle: Transferable Representation and Inference of cell
        cycle
Version: 1.21.0
Authors@R: c(
        person("Shijie", "Zheng", role=c("aut", "cre"), email="shijieczheng@gmail.com"))
Description: The package contains functions to infer and visualize cell cycle 
        process using Single Cell RNASeq data. It exploits the idea of transfer
        learning, projecting new data to the previous learned biologically 
        interpretable space. We provide a pre-learned cell cycle space, 
        which could be used to infer cell cycle time of human and mouse single
        cell samples. In addition, we also offer functions to visualize cell
        cycle time on different embeddings and functions to build new reference.
Depends: R (>= 4.0), SingleCellExperiment
Imports: methods, circular, ggplot2, ggnewscale, AnnotationDbi, scater,
        GenomicRanges, IRanges, S4Vectors, scattermore, dplyr,
        RColorBrewer, grDevices, stats, SummarizedExperiment, utils
Suggests: testthat (>= 3.0.0), BiocStyle, knitr, rmarkdown, CircStats,
        cowplot, htmltools, Seurat, org.Hs.eg.db, org.Mm.eg.db
License: GPL-3
VignetteBuilder: knitr
Encoding: UTF-8
LazyData: FALSE
RoxygenNote: 7.1.2
biocViews: SingleCell, Software, Transcriptomics, RNASeq,
        Transcription, BiologicalQuestion, DimensionReduction,
        ImmunoOncology
URL: https://github.com/hansenlab/tricycle
BugReports: https://github.com/hansenlab/tricycle/issues
git_url: https://git.bioconductor.org/packages/tricycle
git_branch: devel
git_last_commit: f021373
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 01:40:26 UTC; biocbuild
Author: Shijie Zheng [aut, cre]
Maintainer: Shijie Zheng <shijieczheng@gmail.com>

# Generated by roxygen2: do not edit by hand

export(circle_scale_legend)
export(diagnose_totalumi)
export(estimate_Schwabe_stage)
export(estimate_cycle_position)
export(fit_periodic_loess)
export(plot_ccposition_den)
export(plot_emb_circle_scale)
export(project_cycle_space)
export(run_pca_cc_genes)
import(ggplot2)
importClassesFrom(AnnotationDbi,AnnotationDb)
importClassesFrom(SingleCellExperiment,SingleCellExperiment)
importClassesFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(AnnotationDbi,mapIds)
importFrom(AnnotationDbi,select)
importFrom(RColorBrewer,brewer.pal)
importFrom(SingleCellExperiment,"reducedDim<-")
importFrom(SingleCellExperiment,altExp)
importFrom(SingleCellExperiment,reducedDim)
importFrom(SingleCellExperiment,reducedDimNames)
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,colData)
importFrom(circular,circular)
importFrom(circular,coord2rad)
importFrom(circular,density.circular)
importFrom(dplyr,.data)
importFrom(dplyr,`%>%`)
importFrom(dplyr,filter)
importFrom(ggnewscale,new_scale_color)
importFrom(grDevices,colorRampPalette)
importFrom(methods,is)
importFrom(scattermore,geom_scattermore)
importFrom(stats,cor)
importFrom(stats,loess)
importFrom(stats,predict)
importFrom(utils,data)
importMethodsFrom(AnnotationDbi,colnames)
importMethodsFrom(AnnotationDbi,get)
importMethodsFrom(AnnotationDbi,ncol)
importMethodsFrom(AnnotationDbi,nrow)
importMethodsFrom(GenomicRanges,intersect)
importMethodsFrom(GenomicRanges,union)
importMethodsFrom(IRanges,"colnames<-")
importMethodsFrom(IRanges,"rownames<-")
importMethodsFrom(IRanges,as.matrix)
importMethodsFrom(IRanges,diff)
importMethodsFrom(IRanges,lapply)
importMethodsFrom(IRanges,rownames)
importMethodsFrom(IRanges,table)
importMethodsFrom(IRanges,tolower)
importMethodsFrom(IRanges,toupper)
importMethodsFrom(IRanges,unique)
importMethodsFrom(IRanges,which)
importMethodsFrom(S4Vectors,"%in%")
importMethodsFrom(S4Vectors,do.call)
importMethodsFrom(S4Vectors,levels)
importMethodsFrom(S4Vectors,t)
importMethodsFrom(SingleCellExperiment,"reducedDim<-")
importMethodsFrom(SingleCellExperiment,altExp)
importMethodsFrom(SingleCellExperiment,cbind)
importMethodsFrom(SingleCellExperiment,rbind)
importMethodsFrom(SingleCellExperiment,reducedDim)
importMethodsFrom(SingleCellExperiment,reducedDimNames)
importMethodsFrom(SummarizedExperiment,assay)
importMethodsFrom(SummarizedExperiment,assayNames)
importMethodsFrom(SummarizedExperiment,colData)
importMethodsFrom(SummarizedExperiment,order)
importMethodsFrom(SummarizedExperiment,sort)
importMethodsFrom(scater,runPCA)

<img src="man/figures/logo.png" align="right" alt="logo.png" width="180" />

# tricycle

<!-- badges: start -->
[![R build status](https://github.com/hansenlab/tricycle/workflows/R-CMD-check-bioc/badge.svg)](https://github.com/hansenlab/tricycle/actions)

<!-- badges: end -->

## Bioconductor release status

|      Branch      |    R CMD check   | Last updated |
|:----------------:|:----------------:|:------------:|
| [_devel_](http://bioconductor.org/packages/devel/bioc/html/tricycle.html) | [![Bioconductor-devel Build Status](http://bioconductor.org/shields/build/devel/bioc/tricycle.svg)](http://bioconductor.org/checkResults/devel/bioc-LATEST/tricycle) | ![](http://bioconductor.org/shields/lastcommit/devel/bioc/tricycle.svg) |
| [_release_](http://bioconductor.org/packages/release/bioc/html/tricycle.html) | [![Bioconductor-release Build Status](http://bioconductor.org/shields/build/release/bioc/tricycle.svg)](http://bioconductor.org/checkResults/release/bioc-LATEST/tricycle) | ![](http://bioconductor.org/shields/lastcommit/release/bioc/tricycle.svg) |

## Installation

```r
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("tricycle", dependencies = TRUE)

#' Fit periodic loess line with circular predictor
#'
#' @description  The function will fit a loess line using cell cycle position and other variables, such as expression levels of a gene
#'  or log-transformed totalUMIs numbers. The circular nature of cell cycle position is taken into account by making 3 copies inside the function.
#'  For convenience, the function will also return a scatter plot with fitted line if needed.
#'
#'
#' @param theta.v The cell cycle position - a numeric vector with range between 0 to 2pi.
#' @param y The response variable - a numeric vector with the same length as \code{theta.v}.
#' @param span The parameter \eqn{\alpha} which controls the degree of smoothing. See \code{\link[stats]{loess}}. Default: 0.3
#' @param length.out The number of data points on the fitted lines to be output in the prediction data.frame. Default: 200
#' @param plot If \code{TRUE}, a \code{ggplot} scatter plot will be included in the output list. The figure will plot y ~ theta.v 
#' with points and the fitted \code{\link[stats]{loess}} line. Default: FALSE
#' @param fig.title The title of the figure. Default: NULL
#' @param point.size  The size of the point in scatter plot used by \code{\link[scattermore]{geom_scattermore}}. Default: 2.1
#' @param point.alpha  The alpha value (transparency) of the point in scatter plot used by \code{\link[scattermore]{geom_scattermore}}. Default: 0.6
#' @param line.size  The size of the fitted line, used by \code{\link[ggplot2]{geom_path}}. Default: 0.8
#' @param line.alpha  The alpha value (transparency) of the fitted line, used by \code{\link[ggplot2]{geom_path}}. Default: 0.8
#' @param color.vars Optional. A vector of categorical variable of the same length of \code{theta.v}, and it will be used to color points in figure. Default: NULL
#' @param color.name The name of the color variables. Used as the name for legend. Default: NULL
#' @param x_lab Title of x-axis. Default: "\eqn{\theta}"
#' @param y_lab Title of y-axis. Default: "y"
#' @param hue.colors The string vector gives custom colors. If not given, the default \code{\link[ggplot2]{scale_color_discrete}} will be used. Default: NULL
#' @param ... Other arguments input to \code{\link[stats]{loess}}.
#'
#' @return A list with the following elements:
#' \itemize{
#'   \item fitted - The fitted vaues on the loess line. A vector of the length of y.
#'   \item residual - The residual values from the fitted loess line, i.e. y - y.fit. A vector of the length of y.
#'   \item pred.df - The prediction \code{data.frame} by uniformly sampling theta from 0 - 2*pi. Names of variables: \code{x} and \code{y}. The number of rows equals to \code{length.out}.
#'   \item loess.o - The fitted loess object.
#'   \item rsquared -  The coefficient of determination R2. Calculated as 1 - residual sum of squares / the total sum of squares.
#'   \item fig - When \code{plot} is \code{TRUE}, a \code{ggplot} scatter plot object will be returned with other items.
#' }
#'
#' @details This function fit a normal loess line, but take the circularity of cell cycle position into account by making \code{theta.v} 3 periods
#' (\code{c(theta.v - 2 * pi, theta.v, theta.v + 2 * pi)}) and repeating y 3 times. Only the fitted values corresponding to original \code{theta.v}
#' will be returned. For convenience, the function will also return a scatter plot with fitted line if needed. 
#' Or user can use \code{pred.df} to visualize the loess line themselves.
#'
#'
#' @name fit_periodic_loess
#' @aliases fit_periodic_loess
#' @seealso
#' \code{\link{estimate_cycle_position}}, for inferring cell cycle position.
#'
#' @author Shijie C. Zheng
#'
#' @examples
#' data(neurosphere_example, package = "tricycle")
#' neurosphere_example <- estimate_cycle_position(neurosphere_example)
#' top2a.idx <- which(rowData(neurosphere_example)$Gene == "Top2a")
#' fit.l <- fit_periodic_loess(neurosphere_example$tricyclePosition,
#'  assay(neurosphere_example, "logcounts")[top2a.idx, ], plot = TRUE)
#' fit.l$fig
NULL

#' @import ggplot2
#' @importFrom stats loess predict
#' @importFrom scattermore geom_scattermore
#' @export
fit_periodic_loess <- function(theta.v, y, span = 0.3, length.out = 200, plot = FALSE,
                          fig.title = NULL, point.size = 2.1, point.alpha = 0.6,
                          line.size = 0.8, line.alpha = 0.8,
                          color.vars = NULL, color.name = NULL, x_lab = "\u03b8",
                          y_lab = "y", hue.colors = NULL, ...) {
    stopifnot("theta.v need to be between 0 - 2pi." = (min(theta.v) >= 0) & (max(theta.v) <= 2 * pi), 
              "The length of theta.v and y should be the same." = length(theta.v) == length(y))
    x <- c(theta.v - 2 * pi, theta.v, theta.v + 2 * pi)
    ss.total <- sum(scale(y, scale = FALSE) ^ 2)
    y <- rep(y, 3)
    loess.o <- loess(y ~ x, span = span, ...)
    fitted.v <- loess.o$fitted[(length(theta.v) + 1):(length(theta.v) * 2)]
    residual.v <- loess.o$residuals[(length(theta.v) + 1):(length(theta.v) * 2)]
    rsquared <- 1 - sum(residual.v ^ 2) / ss.total
    
    pred.x <- seq(0, 2 * pi, length.out = length.out)
    pred.y <- predict(loess.o, newdata = data.frame(x = pred.x))
    pred.df <-  data.frame(x = pred.x, y = pred.y)
    if (plot) {
      if (is.null(fig.title)) fig.title <- paste0("(n=", length(theta.v), ")")
      if (!is.null(color.vars)) {
        color.vars <- factor(color.vars)
        stopifnot("Length of theta.v does not match color.vars" = length(color.vars) == length(theta.v))
        if (!is.null(hue.colors)) {
          stopifnot("Number of colors does not match nlevels of color.vars" = nlevels(factor(color.vars)) == length(hue.colors))
          color_scale <- scale_color_manual(values = hue.colors, name = color.name, limits = levels(color.vars))
        } else {
          color_scale <- scale_color_discrete(name = color.name, limits = levels(color.vars))
        }
        tmp.df <- data.frame(theta = theta.v, y = y, color = color.vars)
        p_aes <- aes_string(x = "theta", y = "y", color = "color")
      } else {
        tmp.df <- data.frame(theta = theta.v, y = y)
        p_aes <- aes_string(x = "theta", y = "y")
        color_scale <- NULL
      }
      fig <- ggplot(data = tmp.df) +
        geom_scattermore(mapping = p_aes, pointsize = point.size, alpha = point.alpha) +
        geom_path(data = pred.df, aes_string(x = "x", y = "y"), size = line.size, alpha = line.alpha) +
        color_scale +
        labs(x = x_lab, y = y_lab, title = fig.title) +
        scale_x_continuous(limits = c(0, 2 * pi), breaks = c(0, pi / 2, pi, 3 * pi / 2, 2 * pi), labels = paste0(c(0, 0.5, 1, 1.5, 2), "\u03C0"), name = "\u03b8") +
        .gg_theme
      return(list(fitted = fitted.v, residual = residual.v, pred.df = pred.df, loess.o = loess.o, rsquared = rsquared, fig = fig))
    }
    
    return(list(fitted = fitted.v, residual = residual.v, pred.df = pred.df, loess.o = loess.o, rsquared = rsquared))
}

context("fit_periodic_loess")



test_that("fit_periodic_loess is working", {
	data(neurosphere_example)
	neurosphere_example <- estimate_cycle_position(neurosphere_example)
	theta.v <- neurosphere_example$tricyclePosition
	y <- assay(neurosphere_example, "logcounts")[1, ]
	
	expect_error(fit_periodic_loess(theta.v, y[-1]))
	expect_equal(length(fit_periodic_loess(theta.v, y)$fitted), length(theta.v))
	expect_equal(length(fit_periodic_loess(theta.v, y)$residual), length(theta.v))
	expect_lte(abs(fit_periodic_loess(theta.v, y)$rsquared), 1)
	n <- sample(100:500, 1)
	expect_equal(nrow(fit_periodic_loess(theta.v, y, length.out = n)$pred.df), n)
	
})

#' Project data into the cell cycle pattern space
#'
#' Project mouse and human single cell RNAseq data into a cell cycle embedding by a pre-learned reference projection matrix.
#'
#' @param x A numeric matrix of **log-expression** values where rows are features and columns are cells.
#' Alternatively, a \linkS4class{SummarizedExperiment} or \linkS4class{SingleCellExperiment} containing such a matrix.
#' @param exprs_values Integer scalar or string indicating which assay of \code{x} contains the **log-expression** values. Default: 'logcounts'
#' @param ref.m A custom reference projection matrix to project the new data, where rows are features and columns are dimensions.
#' Users need to use the same type of \code{gname}(or rownames of \code{x}) as for the \code{ref.m}.
#' If no custom ref.m is given, the internal reference \code{neuroRef} will be used.
#' @param gname Alternative rownames of \code{x}. If provided, this will be used to map genes within \code{x} with genes in \code{ref.m}.
#' If not provided, the rownames of \code{x} will be used instead. Default: NULL
#' @param gname.type The type of gene names as in \code{gname} or rownames of \code{x}. It can be either 'ENSEMBL' or 'SYMBOL'. If the user uses
#' custom \code{ref.m}, this value will have no effect. Default: 'ENSEMBL'
#' @param species The type of species in \code{x}. It can be either 'mouse' or 'human'. If the user uses
#' custom \code{ref.m}, this value will have no effect. Default: 'mouse'
#' @param AnnotationDb An AnnotationDb objects. If the user uses the internal reference to project human data,
#'  and provide rownames in the format of Ensembl IDs, this object will be used to map Ensembl IDs to gene SYMBOLs.
#'  If no AnnotationDb object being given, the function will use \code{\link[org.Hs.eg.db]{org.Hs.eg.db}}.
#' @param altexp String or integer scalar specifying an alternative experiment containing the input data.
#' @param name String specifying the name to be used to store the result in the \code{\link[SingleCellExperiment]{reducedDims}} of the output. Default: 'tricycleEmbedding'
#'
#' @details
#' The function will use pre-learned cell cycle pattern to project new data to show the cell cycle progression. If the user uses internal Neuropshere reference,
#' the expression values must be **log-transformed**. Besides, we would assume the input data has been already preprocessed, library size normalized at least.
#' The projection process is to take sum of weighted mean-centered expression of chosen genes, so the mean expression of a given gene could be affected without library size normalization.
#'
#' @return
#' If the input is a numeric matrix or a \linkS4class{SummarizedExperiment}, a projection matrix with rows cells and column dimensions will be returned.
#' The actual rotation matrix used to project the data is included in the attributes with name 'rotation'.
#'
#' For \linkS4class{SingleCellExperiment}, an updated \linkS4class{SingleCellExperiment} is returned containing projection matrix in \code{\link[SingleCellExperiment]{reducedDims}(..., name)}.
#'
#' @name project_cycle_space
#' @seealso
#' \code{\link{estimate_cycle_position}}, for inferring cell cycle position.
#'
#' @references
#' Zheng SC, et al.
#' \emph{Universal prediction of cell cycle position using transfer learning.}
#' Genome Biology (2022) 23: 41
#' doi:\href{https://doi.org/10.1186/s13059-021-02581-y}{10.1186/s13059-021-02581-y}.
#' 
#' @author Shijie C. Zheng
#'
#' @examples
#' data(neurosphere_example, package = "tricycle")
#' neurosphere_example <- project_cycle_space(neurosphere_example)
#' reducedDimNames(neurosphere_example)
#' head(reducedDim(neurosphere_example, "tricycleEmbedding"))
#' plot(reducedDim(neurosphere_example, "tricycleEmbedding"))
#' names(attributes(reducedDim(neurosphere_example, "tricycleEmbedding")))
NULL




.project_cycle_space <- function(data.m, ref.m = NULL, gname = NULL, gname.type = c("ENSEMBL", "SYMBOL"), species = c("mouse", "human"), AnnotationDb = NULL) {
    species <- match.arg(species)
    gname.type <- match.arg(gname.type)

    if (!is.null(gname)) {
        rownames(data.m) <- gname
    }
    if (is.null(ref.m)) {
        message("No custom reference projection matrix provided. The ref learned from mouse Neuroshpere data will be used.")
        ref.m <- .getRotation(gname.type = gname.type, species = species)

        if (species == "human" & gname.type == "ENSEMBL") {
            message("As the reference data was learned from mouse, we will map the human ENSEMBL id to gene SYMBOL.")
            rownames(data.m) <- .humanSymbol(gname = rownames(data.m), AnnotationDb = AnnotationDb)
        }
    }
    .calProjection(data.m, ref.m)
}

.calProjection <- function(data.m, rotation.m) {
    genes <- intersect(rownames(data.m), rownames(rotation.m))

    if (length(genes) == 0) {
          stop("None genes found in new data. This could be caused by wrong input of rownames type.")
      }
    message(paste0("The number of projection genes found in the new data is ", length(genes), "."))

    rotation.m <- rotation.m[genes, ]
    data.m <- data.m[genes, ]
    projection.m <- scale(t(as.matrix(data.m)), center = TRUE, scale = FALSE) %*% rotation.m
    rownames(projection.m) <- colnames(data.m)
    colnames(projection.m) <- colnames(rotation.m)
    attr(projection.m, "rotation") <- rotation.m

    return(projection.m)
}

.getRotation <- function(gname.type, species) {
    data_env <- new.env(parent = emptyenv())
    data("neuroRef", package = "tricycle", envir = data_env)
    neuroRef <- data_env[["neuroRef"]]
    rotation.m <- as.matrix(neuroRef[, seq_len(2)])
    if (species == "human") {
        rownames(rotation.m) <- neuroRef$SYMBOL
    } else {
        rownames(rotation.m) <- neuroRef[, tolower(gname.type)]
    }
    colnames(rotation.m) <- c("PC1", "PC2")
    return(rotation.m)
}

#' @importFrom AnnotationDbi mapIds
#' @importMethodsFrom AnnotationDbi colnames get ncol nrow
.humanSymbol <- function(gname, AnnotationDb = NULL) {
    AnnotationDb <- .getAnnotationDB(AnnotationDb, species = "human")
    SYMBOL <- AnnotationDbi::mapIds(AnnotationDb, keys = gname, column = "SYMBOL", keytype = "ENSEMBL", multiVals = "first")
    return(SYMBOL)
}


#' @export
#' @rdname project_cycle_space
#' @importFrom methods is
#' @importFrom SingleCellExperiment reducedDim<- altExp
#' @importFrom SummarizedExperiment assay
#' @importClassesFrom SummarizedExperiment SummarizedExperiment
#' @importClassesFrom SingleCellExperiment SingleCellExperiment
#' 
project_cycle_space <- function(x, exprs_values = "logcounts", altexp = NULL, name = "tricycleEmbedding", 
                                ref.m = NULL, gname = NULL, gname.type = c("ENSEMBL", "SYMBOL"), species = c("mouse", "human"), AnnotationDb = NULL) {
  if (is(x, "SingleCellExperiment")) {
    if (!is.null(altexp)) {
      y <- altExp(x, altexp)
    } else {
      y <- x
    }
    reducedDim(x, name) <- .project_cycle_space(assay(y, exprs_values), ref.m = ref.m, gname = gname, gname.type = gname.type, species = species, AnnotationDb = AnnotationDb)
    out <- x
  } else if (is(x, "SummarizedExperiment")) {
    out <- .project_cycle_space(assay(x, exprs_values), ref.m = ref.m, gname = gname, gname.type = gname.type, species = species, AnnotationDb = AnnotationDb)
  } else {
    out <- .project_cycle_space(x, ref.m = ref.m, gname = gname, gname.type = gname.type, species = species, AnnotationDb = AnnotationDb)
  }
  return(out)
}

context("estimate_cycle_position")


test_that("estimate_cycle_position", {
	data(neurosphere_example)
	
	expect_s4_class(estimate_cycle_position(neurosphere_example), "SingleCellExperiment")
	expect_type(estimate_cycle_position(assay(neurosphere_example, "logcounts")), "double")
	
	n <- sample(100:400, 1)
	expect_equal(ncol(estimate_cycle_position(neurosphere_example[, seq_len(n)])), n)

	expect_error(project_cycle_space(neurosphere_example, altexp = "aa"))
	expect_error(project_cycle_space(neurosphere_example, dimred = "aa"))

	
})

context("project_cycle_space")


test_that("project_cycle_space", {
	data(neurosphere_example)
	
	expect_s4_class(project_cycle_space(neurosphere_example), "SingleCellExperiment")
	expect_type(project_cycle_space(assay(neurosphere_example, "logcounts")), "double")
	
	n <- sample(100:400, 1)
	expect_equal(ncol(project_cycle_space(neurosphere_example[, seq_len(n)])), n)
	
	expect_error(project_cycle_space(neurosphere_example, gname.type = "SYMBOL"))
	expect_message(project_cycle_space(neurosphere_example, AnnotationDb = "aa"))
	expect_error(project_cycle_space(neurosphere_example, species = "human", AnnotationDb = "aa"), "The input AnnotationDB is not an AnnotationDb object.")
	
	
})

#' Assign cell cycle position
#'
#' Assign cell cycle position by the angle formed by PC1 and PC2 in the cell cycle space.
#' If the cell cycle projection does not exist, the function will project the data.
#'
#' @param x A numeric matrix of **log-expression** values where rows are features and columns are cells.
#' Alternatively, a \linkS4class{SummarizedExperiment} or \linkS4class{SingleCellExperiment} containing such a matrix.
#' @param exprs_values Integer scalar or string indicating which assay of \code{x} contains the **log-expression** values, which will be used for projection.
#' If the projection already exists, you can ignore this value. Default: 'logcounts'
#' @param ... Arguments to be used by \code{\link{project_cycle_space}}. If \code{x} is a \linkS4class{SingleCellExperiment}, and the projection is
#' already in the reducedDim with name \code{dimred}. The \code{dimred} will be directly used to assign cell cycle position withou new projection.
#' @param dimred The name of reducedDims in  \linkS4class{SingleCellExperiment} (\code{\link[SingleCellExperiment]{reducedDims}}). If the \code{dimred} already exists,
#' it will be used to assign cell cycle position. If \code{dimred} does not exist, the projection will be calculated first by \code{\link{project_cycle_space}}
#' and stored with name \code{dimred} in {x}. Default: 'tricycleEmbedding'
#' @param center.pc1 The center of PC1 when defining the angle. Default: 0
#' @param center.pc2 The center of PC2 when defining the angle. Default: 0
#' @param altexp String or integer scalar specifying an alternative experiment containing the **log-expression** data, which will be used for projection.
#' If the projection is already calculated and stored in the \linkS4class{SingleCellExperiment} as a dimred, leave this value to default NULL.
#'
#' @details
#' The function will use assign the cell cycle position by the angle formed by the PC1 and PC2 of cell cycle projections.
#' If the input is a numeric matrix or a \linkS4class{SummarizedExperiment}, the projection will be calculated with the input **log-expression** values.
#' For \linkS4class{SingleCellExperiment}, the projection will also be calculated if the designated \code{dimred} does not exist.
#' Ohterwise, the \code{dimred} will directly be used to assign cell cycle position.
#' Therefore, this function is a wrapper function if the input is a \linkS4class{SingleCellExperiment}.
#' Refer to \code{\link{project_cycle_space}} to all arguments during the projection.
#' 
#' The estimated cell cycle position is bound between 0 and 2pi.
#' Note that we strive to get high resolution of cell cycle state, and we think
#' the continuous position is more appropriate when describing the cell cycle.
#' However, to help users understand the position variable, we also note that 
#' users can approximately relate 0.5pi to be the start of S stage, pi to be the
#' start of G2M stage, 1.5pi to be the middle of M stage, and 1.75pi-0.25pi to 
#' be G1/G0 stage.
#'
#' @return
#' If the input is a numeric matrix, the cell cycle position - a numeric vector bound between \eqn{0 \sim 2 \pi} with the same length as the number of input coumlum will be returned.
#'
#' If the input is \linkS4class{SummarizedExperiment}, the original \linkS4class{SummarizedExperiment} with cell cycle position stored in colData with name 'tricyclePosition' will be returned.
#'
#' If the input is \linkS4class{SingleCellExperiment}, the original \linkS4class{SingleCellExperiment} with cell cycle position stored in colData with name 'tricyclePosition' will be returned
#' and the projection will be stored in \code{\link[SingleCellExperiment]{reducedDims}(..., dimred)} if it does not exist before.
#'
#' @name estimate_cycle_position
#' @seealso
#' \code{\link{project_cycle_space}}, for projecting new data with a pre-learned reference
#'
#' @references
#' Zheng SC, et al.
#' \emph{Universal prediction of cell cycle position using transfer learning.}
#' Genome Biology (2022) 23: 41
#' doi:\href{https://doi.org/10.1186/s13059-021-02581-y}{10.1186/s13059-021-02581-y}.
#'
#' @author Shijie C. Zheng
#'
#' @examples
#' data(neurosphere_example, package = "tricycle")
#' neurosphere_example <- estimate_cycle_position(neurosphere_example)
#' reducedDimNames(neurosphere_example)
#' plot(reducedDim(neurosphere_example, "tricycleEmbedding"))
#' plot(neurosphere_example$tricyclePosition,
#'  reducedDim(neurosphere_example, "tricycleEmbedding")[, 1])
#' plot(neurosphere_example$tricyclePosition,
#'  reducedDim(neurosphere_example, "tricycleEmbedding")[, 2])
NULL

#' @importFrom circular coord2rad
.getTheta <- function(pc1pc2.m, center.pc1 = 0, center.pc2 = 0) {
    pc1pc2.m[, 1] <- pc1pc2.m[, 1] - center.pc1
    pc1pc2.m[, 2] <- pc1pc2.m[, 2] - center.pc2
    as.numeric(coord2rad(pc1pc2.m[, seq_len(2)]))
}


#' @export
#' @rdname estimate_cycle_position
#' @importFrom methods is
#' @importFrom SingleCellExperiment reducedDim<- altExp reducedDimNames reducedDim
#' @importFrom SummarizedExperiment assay
#' @importClassesFrom SummarizedExperiment SummarizedExperiment
#' @importClassesFrom SingleCellExperiment SingleCellExperiment
#' 
estimate_cycle_position <- function(x, exprs_values = "logcounts", dimred = "tricycleEmbedding", center.pc1 = 0, center.pc2 = 0, altexp = NULL, ...) {
    if (is(x, "SingleCellExperiment")) {
        if (!is.null(altexp)) {
            y <- altExp(x, altexp)
            if ((dimred %in% reducedDimNames(x)) & (!(dimred %in% reducedDimNames(y)))) {
                warning(paste0(
                    dimred, "exists in x, but not in ", altexp, ". \nThe projecion will be recalculated using ", altexp, ".\n If you intend to use pre-calculated projection, please don't set ",
                    altexp
                ))
            }
        } else {
            y <- x
        }
        if (!(dimred %in% reducedDimNames(y))) {
            message(paste0("The designated dimred do not exist in the SingleCellExperiment or in altexp. project_cycle_space will be run to calculate embedding ", dimred))
            y <- project_cycle_space(y, name = dimred, exprs_values = exprs_values, ...)
            reducedDim(x, dimred) <- reducedDim(y, dimred)
        }
        x$tricyclePosition <- .getTheta(reducedDim(x, dimred), center.pc1 = center.pc1, center.pc2 = center.pc2)
        out <- x
    } else if (is(x, "SummarizedExperiment")) {
        projection.m <- .project_cycle_space(assay(x, exprs_values), ...)
        x$tricyclePosition <- .getTheta(projection.m, center.pc1 = center.pc1, center.pc2 = center.pc2)
        out <- x
    } else {
        projection.m <- .project_cycle_space(x, ...)
        out <- .getTheta(projection.m, center.pc1 = center.pc1, center.pc2 = center.pc2)
    }
    return(out)
}

#' 5 stage cell cycle gene marker list from Revelio
#'
#' This 5 stage cell cycle gene marker list is directly from \code{Revelio} package.
#' Within the list, 5 vectors corresponds to highly expressed genes at the cell cycle stage.
#' The genes are given as the human gene SYMBOLS.
#' This gene list is originally from the Whitfield et al.(2020).
#'
#' @name RevelioGeneList
#' @docType data
#'
#' @usage data(RevelioGeneList)
#'
#' @format An list of 5 string vector. The names of the elements are the names of cell cycle stages with the order of:
#' G1S, S, G2, G2M, MG1.
#'
#' @keywords datasets
#'
#' @references
#' Whitfield ML, et al.
#' \emph{Identification of Genes Periodically Expressed in the Human Cell Cycle and Their Expression in Tumors.}
#' Molecular Biology of the Cell (2002) 13: 1977-2000
#' doi:\href{https://doi.org/10.1091/mbc.02-02-0030}{10.1091/mbc.02-02-0030}.
#'
#' @importFrom utils data
#' @examples
#' data(RevelioGeneList)
NULL

#' Assign cell cycle stages using Schwabe method
#'
#' The function is a re-implementation of cell cycle stage assignment method
#' proposed in Schwabe et al.(2020), with a little modification. The core
#'  assignment method is not designed by the authors of this package!
#'
#' @param x A numeric matrix of **log-expression** values where rows are features and columns are cells.
#' Alternatively, a \linkS4class{SummarizedExperiment} or \linkS4class{SingleCellExperiment} containing such a matrix.
#' @param exprs_values Integer scalar or string indicating which assay of \code{x} contains the **log-expression** values, which will be used for projection.
#' If the projection already exists, you can ignore this value. Default: 'logcounts'
#' @param batch.v A string specifies which column in colData of \linkS4class{SummarizedExperiment} or \linkS4class{SingleCellExperiment} to use as the batch variable.
#' Or it can be a vector, of which the number of elements equals to the number of columns of \code{x}. The 5 stage cell cycle assignments are preformed for each batch separately.
#' No \code{NA} is permitted. Default: NULL
#' @param cycleGene.l A list contains the marker genes for each stage. The stage names should be included as names of the elements. If user feed custom list,
#' they should make sure that the same gene id type for \code{x} and \code{cycleGene.l}. If not custom list is given, \code{\link{RevelioGeneList}} will be used.
#' Default: NULL
#' @param gname Alternative rownames of \code{x}. If provided, this will be used to map genes within \code{x} with genes in \code{ref.m}.
#' If not provided, the rownames of \code{x} will be used instead. Default: NULL
#' @param gname.type The type of gene names as in \code{gname} or rownames of \code{x}. It can be either 'ENSEMBL' or 'SYMBOL'. If the user uses
#' custom \code{ref.m}, this value will have no effect. Default: 'ENSEMBL'
#' @param species The type of species in \code{x}. It can be either 'mouse' or 'human'. If the user uses
#' custom \code{cycleGene.l}, this value will have no effect. Default: 'mouse'
#' @param corThres For each batch and each stage, correlations between expression of each gene and the mean of all genes belonging to that stage
#' will be calculated to filter the final gene list used for inference. The genes with a correlation between \code{corThres} will not be used for calculating \emph{z}-scores.
#' Default: 0.2
#' @param tolerance For each cell, the function will compare the largest two \emph{z}-scores. If the difference between those two \emph{z}-scores is less than \code{tolerance},
#' the cell will be treated un-assignable with \code{NA} value returned for that cell. Default: 0.3
#' @param AnnotationDb An AnnotationDb objects. It is used to map ENSEMBL IDs to gene SYMBOLs.
#'  If no AnnotationDb object being given, the function will use \code{\link[org.Hs.eg.db]{org.Hs.eg.db}} or \code{\link[org.Mm.eg.db]{org.Mm.eg.db}} for human and mouse respectively.
#' @param altexp String or integer scalar specifying an alternative experiment containing the **log-expression** data, which will be used for projection.
#' If the projection is already calculated and stored in the \linkS4class{SingleCellExperiment} as a dimred, leave this value to default NULL.
#'
#' @details
#' The function is a re-implementation of cell cycle stage assignment method
#' proposed in Schwabe et al.(2020), with a little modification. We include 
#' this function only for the purpose of convenience. The core assignment method
#' is not designed by the authors of this package!
#' Breiefly, the function assigns cells to discretized cell cycle stages by
#'  comparing the \emph{z}-scores calculated for each stage markers.
#' Without cycleGene.l input, \code{\link{RevelioGeneList}} will be used.
#' If you use this function, you should cite Schwabe et al.(2020).
#'
#' @return
#' If the input is a numeric matrix, the discretized cell cycle stages - a factor vector corresponding to each cell will be returned.
#'
#' If the input is \linkS4class{SummarizedExperiment}, the original \linkS4class{SummarizedExperiment} with the discretized cell cycle stages stored in colData with name 'CCStage' will be returned.
#'
#' If the input is \linkS4class{SingleCellExperiment}, the original \linkS4class{SingleCellExperiment} with the discretized cell cycle stages stored in colData with name 'CCStage' will be returned.
#'
#' @name estimate_Schwabe_stage
#' @aliases estimate_Schwabe_stage
#'
#'
#' @author Shijie C. Zheng
#'
#' @references
#' Schwabe D, et al.
#' \emph{The transcriptome dynamics of single cells during the cell cycle.}
#' Molecular Systems Biology (2020) 16: e9946
#' doi:\href{https://doi.org/10.15252/msb.20209946}{10.15252/msb.20209946}.
#'
#' Zheng SC, et al.
#' \emph{Universal prediction of cell cycle position using transfer learning.}
#' Genome Biology (2022) 23: 41
#' doi:\href{https://doi.org/10.1186/s13059-021-02581-y}{10.1186/s13059-021-02581-y}.
#' 
#' @examples
#' data(neurosphere_example, package = "tricycle")
#' neurosphere_example <- estimate_Schwabe_stage(neurosphere_example,
#'  gname.type = "ENSEMBL", species = "mouse")
#' neurosphere_example2 <- estimate_Schwabe_stage(neurosphere_example, batch.v = "sample")
#' neurosphere_example3 <- estimate_Schwabe_stage(neurosphere_example,
#'  batch.v = neurosphere_example$sample)
#' neurosphere_example <- project_cycle_space(neurosphere_example)
#' plot(reducedDim(neurosphere_example, "tricycleEmbedding"),
#'  col = neurosphere_example$CCStage)
NULL


#' @importFrom stats cor
#' @importMethodsFrom S4Vectors  "%in%"  do.call  levels  t
.CCStage <- function(data.m, batch.v = NULL, cycleGene.l = NULL, corThres = 0.2, tolerance = 0.3) {
    if (is.null(batch.v)) {
          batch.v <- rep(1, ncol(data.m))
      }
    if ((ncol(data.m) != length(batch.v)) | any(is.na(batch.v))) {
          stop("The length of batch.v does not equal to the number of column in
               data or there is NA in bathc.v, which is not allowed.")
      }
    cycleGene.l <- lapply(cycleGene.l, intersect, rownames(data.m))
    allgenes.v <- Reduce(union, cycleGene.l)
    if (sum(allgenes.v %in% rownames(data.m)) < 30) {
          stop("Less than 30 cell cycle gene found. Not enough infomation to assgin the 5 stages.")
      }
    data.m <- as.matrix(data.m[allgenes.v, ])

    cc.v <- rep(NA_character_, ncol(data.m))
    for (b in unique(batch.v)) {
        idx <- which(batch.v == b)
        dat <- data.m[, idx]
        score.m <- t(scale(t(scale(do.call(cbind, lapply(seq_along(cycleGene.l), function(i) {
            gene <- cycleGene.l[[i]]
            gene.sum <- rowSums(dat[gene, ])
            if (sum(gene.sum > 0) > 3) {
                gene <- gene[which(gene.sum > 0)]
            } else {
                stop(paste0("Less than 3 ", names(cycleGene.l)[i], " stage genes expressed in bacth ", b, ". \n Consider changing batch setting."))
            }
            mean.v <- colMeans(dat[gene, ])
            cor.v <- cor(as.matrix(t(dat[gene, ])), as.vector(mean.v))
            if (sum(cor.v > corThres) < 3) {
                  warning(paste0("Batch ", b, " phase ", names(cycleGene.l)[i], " too few genes (<3)."))
              }
            message(paste0("Batch ", b, " phase ", names(cycleGene.l)[i], " gene:", sum(cor.v > corThres), "\n"))
            return(colMeans(dat[gene[cor.v > corThres], ]))
        }))))))
        cc.v[idx] <- apply(score.m, MARGIN = 1, function(s) {
            o.v <- order(s, decreasing = TRUE)
            eta <- o.v[1]
            eta_ <- o.v[2]
            if (((abs(eta - eta_) > 1) & (abs(eta - eta_) < (length(cycleGene.l) - 1))) | ((s[eta] - s[eta_]) < tolerance)) {
                  return(NA_character_)
              }
            return(names(cycleGene.l)[eta])
        })
    }
    cc.v <- factor(cc.v, levels = names(cycleGene.l))
    return(cc.v)
}


#' @importFrom AnnotationDbi mapIds
#' @importMethodsFrom AnnotationDbi colnames get ncol nrow
#' @importMethodsFrom GenomicRanges intersect union
.getSYMBOL <- function(gname, species = c("mouse", "human"), AnnotationDb = NULL) {
    species <- match.arg(species)
    AnnotationDb <- .getAnnotationDB(AnnotationDb, species)
    SYMBOL <- suppressMessages(toupper(AnnotationDbi::mapIds(AnnotationDb, keys = gname, column = "SYMBOL", keytype = "ENSEMBL", multiVals = "first")))
    return(SYMBOL)
}

#' @importMethodsFrom IRanges as.matrix "colnames<-"  diff lapply rownames "rownames<-"  table tolower toupper unique which
.estimate_Schwabe_stage <- function(data.m, cycleGene.l = NULL, gname = NULL, gname.type = c("ENSEMBL", "SYMBOL"), species = c("mouse", "human"), AnnotationDb = NULL, ...) {
    species <- match.arg(species)
    gname.type <- match.arg(gname.type)
    if (is.null(gname)) {
        message("No gname input. Rownames of x will be used.")
        gname <- rownames(data.m)
    } else {
        if (nrow(data.m) != length(gname)) stop("gname does not match nrow x!")
    	  rownames(data.m) <- gname
    }
    if (is.null(cycleGene.l)) {
        if (species == "mouse") {
            if (gname.type == "SYMBOL") {
                rownames(data.m) <- toupper(rownames(data.m))
            } else {
                rownames(data.m) <- .getSYMBOL(gname, species = "mouse", AnnotationDb = AnnotationDb)
            }
        } else {
            if (gname.type == "ENSEMBL") {
                  rownames(data.m) <- .getSYMBOL(gname, species = "human", AnnotationDb = AnnotationDb)
              }
        }
        data_env <- new.env(parent = emptyenv())
        data("RevelioGeneList", package = "tricycle", envir = data_env)
        cycleGene.l <- data_env[["RevelioGeneList"]]
    }

    return(.CCStage(data.m, cycleGene.l = cycleGene.l, ...))
}


#' @export
#' @rdname estimate_Schwabe_stage
#' @importMethodsFrom SummarizedExperiment  assay colData  order sort
#' @importMethodsFrom SingleCellExperiment altExp cbind rbind reducedDim "reducedDim<-"  reducedDimNames
#' 
estimate_Schwabe_stage <- function(x, exprs_values = "logcounts", batch.v = NULL,
                                   altexp = NULL, cycleGene.l = NULL, gname = NULL,
                                   gname.type = c("ENSEMBL", "SYMBOL"),
                                   species = c("mouse", "human"), AnnotationDb = NULL,
                                   corThres = 0.2, tolerance = 0.3) {
	message("This function is a re-implementation of Schwabe et al. 2020. If you want to use tricycle method, please run estimate_cycle_position!")
  if (is(x, "SingleCellExperiment")) {
    if (!is.null(batch.v)) {
      if ((length(batch.v) == 1) & all(batch.v %in% names(colData(x)))) {
        batch.v <- colData(x)[, batch.v]
      }
    }
    if (!is.null(altexp)) {
      y <- altExp(x, altexp)
    } else {
      y <- x
    }
    x$CCStage <- .estimate_Schwabe_stage(assay(y, exprs_values), batch.v = batch.v,
                                         cycleGene.l = cycleGene.l, gname = gname,
                                         gname.type = gname.type, species = species,
                                         AnnotationDb = AnnotationDb, corThres = corThres,
                                         tolerance = tolerance)
    out <- x
  } else if (is(x, "SummarizedExperiment")) {
    if (!is.null(batch.v)) {
      if ((length(batch.v) == 1) & all(batch.v %in% names(colData(x)))) {
        batch.v <- colData(x)[, batch.v]
      }
    }
    x$CCStage <- .estimate_Schwabe_stage(assay(x, exprs_values), batch.v = batch.v,
                                         cycleGene.l = cycleGene.l, gname = gname,
                                         gname.type = gname.type, species = species,
                                         AnnotationDb = AnnotationDb, corThres = corThres,
                                         tolerance = tolerance)
    out <- x
  } else {
    out <- .estimate_Schwabe_stage(x, batch.v = batch.v,
                                   cycleGene.l = cycleGene.l, gname = gname,
                                   gname.type = gname.type, species = species,
                                   AnnotationDb = AnnotationDb, corThres = corThres,
                                   tolerance = tolerance)
  }
  return(out)
}

#' Pre-learned reference projection matrix from the Neurosphere dataset
#'
#' Default reference projection matrix learned from the Neurosphere dataset.
#'
#' @name neuroRef
#' @docType data
#'
#' @usage data(neuroRef)
#'
#' @format An object of class `'data.frame'`, with 5 variables. Normally, user won't call this
#' data directly as it will be automatically used in \code{\link{project_cycle_space}} if no custom reference
#' projection matrix is provided. Each row is gene, and rotation scores for PC1 and PC2, mouse ENSEMBL IDs,
#' and mouse gene SYMBOLs are included. The `SYMBOL` values are just the upper-case `symbol` values.
#'
#' @keywords datasets
#'
#' @references
#' Zheng SC, et al.
#' \emph{Universal prediction of cell cycle position using transfer learning.}
#' Genome Biology (2022) 23: 41
#' doi:\href{https://doi.org/10.1186/s13059-021-02581-y}{10.1186/s13059-021-02581-y}.
#'
#' @importFrom utils data
#' @examples
#' data(neuroRef)
NULL

.gg_theme <- theme_bw(base_size = 8) +
    theme(
        panel.grid.minor = element_blank(),
        panel.grid.major.x = element_blank(),
        panel.grid.major.y = element_line(size = 0.25),
        axis.ticks = element_line(colour = "black", linetype = 1, size = 0.25),
        axis.text.y = element_text(size = 6),
        axis.text.x = element_text(size = 6, vjust = 0.5, hjust = 0.5),
        axis.title.y = element_text(size = 7),
        axis.title.x = element_text(size = 7),
        plot.title = element_text(face = "plain", size = 8, hjust = 0.5),
        plot.margin = unit(c(5, 5, 6, 6), "pt"),
        legend.background = element_blank(),
        legend.position = "right",
        legend.justification = c(0.5, 0.5),
        legend.key.size = unit(7, "pt"),
        legend.key = element_blank(),
        legend.margin = margin(t = 1, r = 2, b = 2, l = 2, unit = "pt"),
        legend.title = element_text(size = 5),
        legend.title.align = 0.5,
        legend.text.align = 0.5,
        legend.text = element_text(size = 5),
        strip.text = element_text(color = "white"),
        strip.background = element_rect(fill = "black", color = "black")
    )


#' @importFrom methods is
#' @importClassesFrom AnnotationDbi AnnotationDb
.getAnnotationDB <- function(AnnotationDB = NULL, species = c("mouse", "human")) {
    species <- match.arg(species)
    if (!is.null(AnnotationDB)) {
        stopifnot("The input AnnotationDB is not an AnnotationDb object.", is(AnnotationDB, "AnnotationDb"))
    } else {
        if (species == "mouse") {
            stopifnot("No AnnotationDB input and org.Mm.eg.db package is not installed. Please install org.Mm.eg.db package." = nzchar(system.file(package = "org.Mm.eg.db")))
            AnnotationDb <- org.Mm.eg.db::org.Mm.eg.db
            message("No AnnotationDb desginated. org.Mm.eg.db will be used to map Mouse ENSEMBL id to gene SYMBOL.")
        } else {
            stopifnot("No AnnotationDB input and org.Hs.eg.db package is not installed. Please install org.Mm.Hs.db package." = nzchar(system.file(package = "org.Hs.eg.db")))
            AnnotationDb <- org.Hs.eg.db::org.Hs.eg.db
            message("No AnnotationDb desginated. org.Hs.eg.db will be used to map Human ENSEMBL id to gene SYMBOL.")
        }
    }
    return(AnnotationDb)
}

#' Plot cell cycle position kernel density stratified by a factor
#'
#' @description  The function will compute and plot cell cycle position kernel density.
#'
#'
#' @param theta.v The cell cycle position - a numeric vector with range between 0 to 2pi.
#' @param color_var.v A factor variable to stratify \code{theta.v}, such as samples or 'CCStage'. The length of it should equal to the length of \code{theta.v}.
#' @param color_name The name of the color_var.v to be used in the legend.
#' @param palette.v A string vector to give the color names. It should has the length of the number of levels of \code{color_var.v}. If not given, the 'Set1' palette will be used.
#'  (See \code{\link[RColorBrewer]{display.brewer.all}}) If the number of levels
#'  of \code{color_var.v} is greater than 8, only the top 8 levels of most cell
#'   will be shown. You can show them all by feeding enough colors in \code{palette.v}. Default: NULL
#' @param fig.title The title of the figure. Default: NULL
#' @param type It can be either of 'linear' or 'circular'. 'linear' corresponds to Cartesian coordinate system and 'circular' for polar coordinate system. Default: 'linear'
#' @param bw The smoothing bandwidth to be used. It is equal to the concentration parameter of the von Mises distribution. See \code{\link[circular]{density.circular}}. Default: 30
#' @param weighted Whether the density should be weighted on the percentage of each level of \code{color_var.v}. Default: FALSE
#' @param line.size The size of the line used by \code{\link[ggplot2]{geom_path}}. Default: 0.5
#' @param line.alpha The alpha value of the line used by \code{\link[ggplot2]{geom_path}}. Default: 0.5
#' @param addRug Whether to add rug on the bottom of the linear density plot or an inner circle on the circular plot to show the continuous scale of theta. Default: TRUE
#' @param RugPalette.v The palette used for the rug plot. If not given, it will used the same default palette as in \code{\link{plot_emb_circle_scale}}.
#' @param ... Other arguments accepted by \code{\link[ggplot2]{geom_path}}.
#'
#' @return A ggplot object
#'
#' @details The function first estimates kernel density using the von Mises distribution. Then,
#' it plots out the density in the polar coordinate system or Cartesian coordinate system. Different colors
#' represents different levels of \code{color_var.v} and the dashed black line is the marginal distribution of all cells.
#'
#'
#'
#' @name plot_ccposition_den
#' @aliases plot_ccposition_den
#' @seealso
#' \code{\link{estimate_Schwabe_stage}}, for inferring 5 stages of cell cycle
#'
#' @author Shijie C. Zheng
#'
#' @examples
#' data(neurosphere_example, package = "tricycle")
#' neurosphere_example <- estimate_cycle_position(neurosphere_example)
#' plot_ccposition_den(neurosphere_example$tricyclePosition, neurosphere_example$sample, "sample")
#'
#' neurosphere_example <- estimate_Schwabe_stage(neurosphere_example,
#'  gname.type = "ENSEMBL", species = "mouse")
#' plot_ccposition_den(neurosphere_example$tricyclePosition, neurosphere_example$CCStage, "CCStage")
NULL





#' @importFrom circular circular density.circular
#' @importFrom RColorBrewer brewer.pal
#' @importFrom ggnewscale new_scale_color
#' @import ggplot2
#'
#' @export
plot_ccposition_den <- function(theta.v, color_var.v, color_name,
																palette.v = NULL, fig.title = NULL,
																type = c("linear", "circular"),
																bw = 30, weighted = FALSE, line.size = 0.5, line.alpha = 0.5,
																addRug = TRUE, RugPalette.v = NULL, 
    ...) {
    if ((min(theta.v) < 0) | (max(theta.v) > 2 * pi)) {
          stop("theta.v need to be between 0 - 2pi.")
      }
    if (length(theta.v) != length(color_var.v)) {
          stop("The length of theta.v and color_var.v should be the same.")
      }
    type <- match.arg(type)

    d <- density.circular(circular(theta.v), bw = bw)
    all.df <- data.frame(x = as.numeric(d$x), y = d$y)

    color_var.v <- factor(color_var.v)
    if (is.null(palette.v)) {
        if (nlevels(color_var.v) > 8) {
            warning("The number of levels of color_var.v is greater than 8.\n Only the top 8 levels of most cell will be shown.\nYou can show them all by feeding enough colors in palette.v.")
            color_var.v[!(color_var.v %in% names(sort(table(color_var.v), decreasing = TRUE))[seq_len(8)])] <- NA
        }
        if (nlevels(color_var.v) == 2) {
            palette.v <- brewer.pal(3, "Set1")[seq_len(2)]
        } else {
            palette.v <- brewer.pal(nlevels(color_var.v), "Set1")
        }
    } else {
        if (nlevels(color_var.v) > length(palette.v)) {
              warning("The number of colors in palette.v doesn't match the number of levels in color_var.v. ")
          }
    }

    if (weighted) {
        weights <- table(color_var.v) / sum(!is.na(color_var.v), na.rm = TRUE)
    } else {
        weights <- rep(1, length(color_var.v))
    }

    if (is.null(fig.title)) {
          fig.title <- paste0("(n=", length(theta.v), ")")
      }
    
    if (any(is.na(color_var.v))) message("NA found in color_var.v. \nThe cells with NA will be considered for global density but not stratified densities.")
    
    strati.df <- do.call(rbind, lapply(seq_len(nlevels(color_var.v)), function(idx) {
        d <- density.circular(circular(theta.v[which(color_var.v == levels(color_var.v)[idx])]), bw = bw)
        return(data.frame(x = as.numeric(d$x), y = d$y * weights[idx], color = levels(color_var.v)[idx]))
    }))
    strati.df$color <- factor(strati.df$color, levels = levels(color_var.v))

    max.v <- max(all.df$y, strati.df$y)


    if (type == "linear") {
        p <- ggplot(strati.df, aes_string(x = "x", y = "y")) +
            geom_path(aes_string(color = "color"), size = line.size, alpha = line.alpha, ...) +
            geom_path(data = all.df, size = line.size, alpha = line.alpha, color = "black", linetype = "dashed", ...) +
            scale_color_manual(values = palette.v, name = color_name, labels = paste0(levels(color_var.v), "\nn=", table(color_var.v)), limits = levels(color_var.v)) +
            scale_x_continuous(limits = c(0, 2 * pi), breaks = c(0, pi / 2, pi, 3 * pi / 2, 2 * pi), labels = paste0(c(0, 0.5, 1, 1.5, 2), "\u03C0"), name = "\u03b8") +
            labs(title = fig.title, y = "Density") +
            .gg_theme
        if (addRug) {
        	if (is.null(RugPalette.v)) RugPalette.v <- c("#2E22EA","#9E3DFB", "#F86BE2", "#FCCE7B", "#C4E416", "#4BBA0F", "#447D87", "#2C24E9")
        	rug.df <- data.frame(x = seq(from = 0, to = 2 * pi, length.out = 200), y = 1)
        	p <- p + 
        		new_scale_color() +
        		geom_rug(data = rug.df, aes_string(x = "x", color = "x"), sides = "b", inherit.aes = FALSE) +
        		scale_color_gradientn(limits = range(0, 2 * pi), breaks = seq(from = 0, to = 2 * pi, length.out = 100), colors = RugPalette.v, guide = "none")
        	
        }
    } else {
        p <- ggplot(strati.df, aes(x = get("x"), y = get("y") + max.v)) +
            geom_path(aes_string(color = "color"), size = line.size, alpha = line.alpha, ...) +
            geom_path(data = all.df, size = line.size, alpha = line.alpha, color = "black", linetype = "dashed", ...) +
            scale_color_manual(values = palette.v, name = color_name, labels = paste0(levels(color_var.v), "\nn=", table(color_var.v)), limits = levels(color_var.v)) +
            coord_polar(theta = "x", start = -pi / 2, direction = -1, clip = "on") +
            scale_x_continuous(limits = c(0, 2 * pi), breaks = c(0, pi / 2, pi, 3 * pi / 2), labels = paste0(c(0, 0.5, 1, 1.5), "\u03C0"), name = "") +
            scale_y_continuous(limits = c(0, max.v * 2), breaks = c(max.v, max.v * 1.5, max.v * 2), labels = c("0", format(max.v * c(0.5, 1), digits = 3)), name = "Density") +
            labs(title = fig.title) +
            .gg_theme
        if (addRug) {
        	if (is.null(RugPalette.v)) RugPalette.v <- c("#2E22EA","#9E3DFB", "#F86BE2", "#FCCE7B", "#C4E416", "#4BBA0F", "#447D87", "#2C24E9")
        	rug.df <- data.frame(x = seq(from = 0, to = 2 * pi, length.out = 200), y = 1)[-1, ]
        	p <- p + 
        		new_scale_color() +
        		geom_rect(data = rug.df, inherit.aes = FALSE, 
        							aes( xmin = x , xmax = x , color = x, fill = x),
        							ymin = max.v * 0.75, ymax = max.v * 0.95, alpha = 0.6) +
        		scale_color_gradientn(limits = range(0, 2 * pi), breaks = seq(from = 0, to = 2 * pi, length.out = 100), colors = RugPalette.v, guide = "none") +
        		scale_fill_gradientn(limits = range(0, 2 * pi), breaks = seq(from = 0, to = 2 * pi, length.out = 100), colors = RugPalette.v, guide = "none")
        	
        }
    }
    
    return(p)
}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----echo=FALSE---------------------------------------------------------------
htmltools::img(src = knitr::image_uri(file.path("../man/figures", "logo.png")),
               alt = 'logo',
               style = 'position:absolute; top:50px; right:5px; padding:10px;height:200px')

## ----setup, message = FALSE---------------------------------------------------
library(tricycle)

## ----example, message=FALSE---------------------------------------------------
data(neurosphere_example)

## ----project, message = TRUE--------------------------------------------------
neurosphere_example <- project_cycle_space(neurosphere_example)
neurosphere_example

## ----plot_projection, message = FALSE-----------------------------------------
library(ggplot2)
library(scattermore)
library(scater)
scater::plotReducedDim(neurosphere_example, dimred = "tricycleEmbedding") +
    labs(x = "Projected PC1", y = "Projected PC2") +
    ggtitle(sprintf("Projected cell cycle space (n=%d)",
                    ncol(neurosphere_example))) +
    theme_bw(base_size = 14)

## ----tricyclePosition, message = TRUE-----------------------------------------
neurosphere_example <- estimate_cycle_position(neurosphere_example)
names(colData(neurosphere_example))

## ----loess, message = TRUE----------------------------------------------------
top2a.idx <- which(rowData(neurosphere_example)$Gene == 'Top2a')
fit.l <- fit_periodic_loess(neurosphere_example$tricyclePosition,
                            assay(neurosphere_example, 'logcounts')[top2a.idx,],
                            plot = TRUE,
                       x_lab = "Cell cycle position \u03b8", y_lab = "log2(Top2a)",
                       fig.title = paste0("Expression of Top2a along \u03b8 (n=",
                                          ncol(neurosphere_example), ")"))
names(fit.l)
fit.l$fig + theme_bw(base_size = 14)

## ----stage, message = TRUE----------------------------------------------------
neurosphere_example <- estimate_Schwabe_stage(neurosphere_example,
                                            gname.type = 'ENSEMBL',
                                            species = 'mouse')
scater::plotReducedDim(neurosphere_example, dimred = "tricycleEmbedding",
                       colour_by = "CCStage") +
  labs(x = "Projected PC1", y = "Projected PC2",
       title = paste0("Projected cell cycle space (n=", ncol(neurosphere_example), ")")) +
  theme_bw(base_size = 14)

## ----density, message = TRUE--------------------------------------------------
plot_ccposition_den(neurosphere_example$tricyclePosition,
                    neurosphere_example$sample, 'sample',
                    bw = 10, fig.title = "Kernel density of \u03b8") +
  theme_bw(base_size = 14)

plot_ccposition_den(neurosphere_example$tricyclePosition,
                    neurosphere_example$sample, 'sample', type = "circular",
                    bw = 10,  fig.title = "Kernel density of \u03b8") +
  theme_bw(base_size = 14)

## ----cyclic, message = TRUE, fig.width = 10, fig.height = 7-------------------
library(cowplot)
p <- plot_emb_circle_scale(neurosphere_example, dimred = 1,
                           point.size = 3.5, point.alpha = 0.9) +
  theme_bw(base_size = 14)
legend <- circle_scale_legend(text.size = 5, alpha = 0.9)
plot_grid(p, legend, ncol = 2, rel_widths = c(1, 0.4))

## ----newRef, message = TRUE---------------------------------------------------
set.seed(100)
gocc_sce.o <- run_pca_cc_genes(neurosphere_example,
                               exprs_values = "logcounts", species = "mouse")
new.ref <- attr(reducedDim(gocc_sce.o, 'PCA'), 'rotation')[, seq_len(2)]
head(new.ref)
new_sce <- estimate_cycle_position(neurosphere_example, ref.m  = new.ref,
                                   dimred = 'tricycleEmbedding2')


## ----cor, message = TRUE------------------------------------------------------
cor(neurosphere_example$tricyclePosition, new_sce$tricyclePosition)
CircStats::circ.cor(neurosphere_example$tricyclePosition, new_sce$tricyclePosition)
qplot(x = neurosphere_example$tricyclePosition,y = new_sce$tricyclePosition) +
  labs(x = "Original \u03b8", y = "New \u03b8",
       title = paste0("Comparison of two \u03b8 (n=", ncol(neurosphere_example), ")")) +
  theme_bw(base_size = 14)

## ----batch, eval = FALSE, echo = TRUE-----------------------------------------
# # suppose we have a count matrix containing all cells across batches; we first subset the matrix to GO cell cycle genes
# library(org.Mm.eg.db)
# library(AnnotationDbi)
# cc.genes <- AnnotationDbi::select(org.Mm.eg.db, keytype = "GOALL",
#                                   keys = "GO:0007049",
#                                   columns = "ENSEMBL")[, "ENSEMBL"]
# count_cc.m <- count.m[ensembl.ids %in% cc.genes, ]  # ensembl.ids is the ensembl.ids for each row of count.m
# 
# # we then construct a Seurat object using the subset matrix and set the batch variable
# library(Seurat)
# seurat.o <- CreateSeuratObject(counts = count_cc.m)
# seurat.o[["batch"]] <- batch.v
# 
# # make a Seurat list and normalize for each batch separately
# # variable features definition is required for FindIntegrationAnchors function
# seurat.list <- lapply(SplitObject(seurat.o, split.by = "batch"),
#                       function(x) FindVariableFeatures(NormalizeData(x)))
# 
# # find anchors and merge data
# seurat.anchors <- FindIntegrationAnchors(object.list = seurat.list)
# seurat.integrated <- IntegrateData(anchorset = seurat.anchors)
# corrected.m <- seurat.integrated@assays$integrated@data
# 
# # run PCA on the batch effects corrected matrix and get the rotaions scores for the top 2 PCs
# pca.m <- scater::calculatePCA(corrected.m, ntop = 500)
# new.ref <- attr(pca.m, 'rotation')[, seq_len(2)]
# 

## ----sessionInfo, echo=FALSE--------------------------------------------------
sessionInfo()

## ----ref, message = FALSE-----------------------------------------------------
data(neuroRef)
head(neuroRef)

#' Example SingleCellExperiment dataset
#'
#' This a subset of mouse Neurosphere data. 200 cells from sample AX1 and AX2 were randonly
#' sampled from the full data. All genes in the GO cell cycle gene list and \code{\link{RevelioGeneList}}
#'  as well as other random 573 genes were included.
#'
#' @name neurosphere_example
#' @docType data
#'
#' @usage data(neurosphere_example)
#'
#' @format A \linkS4class{SingleCellExperiment} object of 1500 genes and 400 cells.
#'
#' @keywords datasets
#'
#'
#' @examples
#' data(neurosphere_example)
NULL

#' Plot embedding with cyclic cell cycle position
#'
#' Generate scat plot of embedding with cyclic cell cycle position or other cyclic variables
#'
#'
#' @param sce.o A \linkS4class{SingleCellExperiment} contains the embbing to be plotted against.
#' @param color_by The name of variable in \code{colData(sce.o)} to be used to show colors. Default: "tricyclePosition"
#' @param facet_by The name of variable in \code{colData(sce.o)} to be used to facet scatter plots. If used, the function will return a list of ggplot objects. If NULL, no faceted panels will be returned. Default: NULL
#' @param dimred The name or index of reducedDims in  \linkS4class{SingleCellExperiment} (\code{\link[SingleCellExperiment]{reducedDims}}). Default: 1
#' @param dim The indices of \code{dimred} to be plotted. At the moment, it has to be two integers.   Default: 1:2
#' @param fig.title The title of the figure. Default: NULL
#' @param point.size  The size of the point in scatter plot used by \code{\link[scattermore]{geom_scattermore}}. Default: 2.1
#' @param point.alpha  The alpha value (transparency) of the point in scatter plot used by \code{\link[scattermore]{geom_scattermore}}. Default: 0.6
#' @param x_lab Title of x-axis. If not given, the colname of \code{dimred} will be used. Default: NULL
#' @param y_lab Title of y-axis. If not given, the colname of \code{dimred} will be used. Default: NULL
#' @param hue.colors The string vector gives the cyclic colors. The first color should look very similar to the last one.
#' Default: c("#2E22EA", "#9E3DFB", "#F86BE2", "#FCCE7B", "#C4E416", "#4BBA0F", "#447D87", "#2C24E9")
#' @param hue.n The number of breaks of color scheme. Default: 500
#' @param plot.legend Whether the legend should be plotted with the scatter plot. We recommend not to use this legend but use the cyclic legend produced by \code{\link{circle_scale_legend}} instead. Default: FALSE
#'
#' @details
#' This function help users plot embedding scater plot colored by cyclic variables, such as cell cycle position, which is bound between 0 - 2pi.
#' It will take a \linkS4class{SingleCellExperiment} object as input, and plot out its \code{dimred} such as PCA, UMAP, and etc with a cyclic color scheme.
#'
#' @return
#' A ggplot object or a list of ggplot objects.
#' If \code{facet_by} is not assigned, a single ggplot plot of the scatter plot will be return,
#' Otherwise, apart from the first scatter plot showing all cells together, other faceted scatter plots will also be given in the list.
#'
#' @name plot_emb_circle_scale
#'
#' @author Shijie C. Zheng
#'
#' @examples
#' data(neurosphere_example, package = "tricycle")
#' neurosphere_example <- estimate_cycle_position(neurosphere_example)
#' plot_emb_circle_scale(neurosphere_example, point.size = 3.1, point.alpha = 0.8)
NULL


#' @importFrom scattermore geom_scattermore
#' @importFrom dplyr filter .data `%>%`
.plot_emb_circle_scale <- function(emb.m, color.value, color_by, facet_var = NULL, fig.title = NULL, point.size = 2.1, point.alpha = 0.6, x_lab = NULL, y_lab = NULL, hue.colors = c(
        "#2E22EA",
        "#9E3DFB", "#F86BE2", "#FCCE7B", "#C4E416", "#4BBA0F", "#447D87", "#2C24E9"
    ), hue.n = 500, plot.legend = FALSE) {
    if (is.null(fig.title)) {
          fig.title <- paste0("(n=", nrow(emb.m), ")")
      }

    if (is.null(x_lab)) {
          x_lab <- colnames(emb.m)[1]
      }
    if (is.null(y_lab)) {
          y_lab <- colnames(emb.m)[2]
      }

    x_lim <- range(emb.m[, 1]) + c(diff(range(emb.m[, 1])) * c(-0.05, 0.05))
    y_lim <- range(emb.m[, 2]) + c(diff(range(emb.m[, 2])) * c(-0.05, 0.05))

    tmp.df <- data.frame(x = emb.m[, 1], y = emb.m[, 2], color = color.value)

    scat.p <- ggplot(tmp.df, aes_string(x = "x", y = "y", color = "color")) +
        geom_scattermore(pointsize = point.size, alpha = point.alpha) +
        scale_color_gradientn(name = color_by, limits = range(0, 2 * pi), breaks = seq(from = 0, to = 2 * pi, length.out = hue.n), colors = hue.colors, guide = "none") +
        labs(y = y_lab, x = x_lab, title = fig.title) +
        xlim(x_lim) +
        ylim(y_lim) +
        .gg_theme

    if (!is.null(facet_var)) {
        tmp.df$facet <- facet_var
        facet_labels <- levels(factor(tmp.df$facet))
        lp <- lapply(seq_len(nlevels(factor(tmp.df$facet))), function(idx) {
            p <- ggplot(tmp.df, aes_string(x = "x", y = "y", color = "color")) +
                geom_scattermore(data = tmp.df %>% dplyr::filter(.data[["facet"]] != levels(factor(tmp.df$facet))[idx]), pointsize = point.size, alpha = 0.4, color = "gray90", show.legend = FALSE) +
                geom_scattermore(data = tmp.df %>% dplyr::filter(.data[["facet"]] == levels(factor(tmp.df$facet))[idx]), pointsize = point.size, alpha = point.alpha) +
                scale_color_gradientn(name = color_by, limits = range(0, 2 * pi), breaks = seq(from = 0, to = 2 * pi, length.out = hue.n), colors = hue.colors, guide = "none") +
                labs(y = y_lab, x = x_lab, title = facet_labels[idx]) +
                xlim(x_lim) +
                ylim(y_lim) +
                .gg_theme
            return(p)
        })
        return(c(list(scat.p), lp))
    }
    return(scat.p)
}



#' @rdname plot_emb_circle_scale
#' 
#' @importFrom AnnotationDbi select
#' @importFrom methods is
#' @importFrom SingleCellExperiment reducedDim
#' @importFrom SummarizedExperiment colData
#' @importClassesFrom SingleCellExperiment SingleCellExperiment
#' @export
#' 
plot_emb_circle_scale <- function(sce.o, color_by = "tricyclePosition", facet_by = NULL, dimred = 1, dim = seq_len(2),
    fig.title = NULL, point.size = 2.1, point.alpha = 0.6, x_lab = NULL, y_lab = NULL,
    hue.colors = c("#2E22EA", "#9E3DFB", "#F86BE2", "#FCCE7B", "#C4E416", "#4BBA0F", "#447D87", "#2C24E9"),
    hue.n = 500, plot.legend = FALSE) {
  stopifnot("sce.o must be a SingleCellExperiment objet." = is(sce.o, "SingleCellExperiment"),
            "The function can only plot 2 dims at this time. Change dim argument." = length(dim) == 2,
            "The color_by variable does not exist in colData(sce.o)." = (color_by %in% names(colData(sce.o))))
  
  
    emb.m <- reducedDim(sce.o, dimred)[, dim]
    color.value <- colData(sce.o)[[color_by]]
    if (!is.null(facet_by)) {
        if (!(facet_by %in% names(colData(sce.o)))) {
              stop("facet_by variable does not exist in colData(sce.o).")
          }
        facet_var <- colData(sce.o)[[facet_by]]
    } else {
        facet_var <- NULL
    }
    .plot_emb_circle_scale(
        emb.m = emb.m, color.value = color.value, color_by = color_by,
        facet_var = facet_var, fig.title = fig.title, point.size = point.size,
        point.alpha = point.alpha, x_lab = x_lab, y_lab = y_lab,
        hue.colors = hue.colors,
        hue.n = hue.n, plot.legend = plot.legend
    )
}



#'
#' @title Get the cyclic legend
#'
#' @description This function is a helper function to create the cyclic ggplot color legend.
#'
#' @usage circle_scale_legend(hue.colors = c("#2E22EA", "#9E3DFB", "#F86BE2", "#FCCE7B", "#C4E416", "#4BBA0F", "#447D87", "#2C24E9"),
#'  hue.n = 500, alpha = 0.6, y.inner = 1.5, y.outer = 3, y.text = 3.8, ymax = 4.5, text.size = 3, addStageLabel = FALSE, 
#'  G1.pos = 0, S.pos = 2.2, G2M.pos = 3.9)
#'
#' @param hue.colors The string vector gives the cyclic colors. The first color should look very similar to the last one.
#' Default: c("#2E22EA", "#9E3DFB", "#F86BE2", "#FCCE7B", "#C4E416", "#4BBA0F", "#447D87", "#2C24E9")
#' @param hue.n The number of breaks of color scheme. Default: 500
#' @param alpha The alpha value (transparency). Default: 0.6
#' @param y.inner The radius of inner circle of the donut. Default: 1.5
#' @param y.outer The radius of outer circle of the donut. Default: 3
#' @param y.text The radius of text position. Default: 3.8
#' @param ymax The value control the border of the legend. Default: 4.5
#' @param text.size The size of the text Default: 3
#' @param addStageLabel Whether to add approximate discrete stage labels. Default: FALSE
#' @param G1.pos Approximate radius value of G1 label position. Default: 0
#' @param S.pos Approximate radius value of S label position. Default: 2.2
#' @param G2M.pos Approximate radius value of G2M label position. Default: 3.9
#'
#' @return A ggplot object
#'
#' @details The function will make a donut shape to serve as the cyclic color legend. The arguments should match the argument used in \code{\link{plot_emb_circle_scale}}.
#'
#' @author Shijie C. Zheng
#'
#' @name circle_scale_legend
#' @aliases circle_scale_legend
#' @rdname circle_scale_legend
#'
#' @examples
#' circle_scale_legend()
NULL


#' @importFrom grDevices colorRampPalette
#' @import ggplot2
#' @export
circle_scale_legend <- function(hue.colors = c("#2E22EA", "#9E3DFB", "#F86BE2",
																							 "#FCCE7B", "#C4E416", "#4BBA0F",
																							 "#447D87", "#2C24E9"),
																hue.n = 500, alpha = 0.6, y.inner = 1.5, y.outer = 3,
																y.text = 3.8, ymax = 4.5, text.size = 3,
																addStageLabel = FALSE, G1.pos = 0, S.pos = 2.2,
																G2M.pos = 3.9) {
	hues.df <- data.frame(theta = seq(from = 0, to = 2 * pi, length.out = hue.n),
												colors = colorRampPalette(hue.colors)(hue.n))
	hue_text.df <- data.frame(theta = c(0, 0.5 * pi, pi, 1.5 * pi),
														label = c("0/2\u03C0", "0.5\u03C0", "\u03C0", "1.5\u03C0"),
														hjust = c(0.1, 0.5, 0.5, 0.5), 
														y.text = rep(y.text, 4))
	legend.p <- ggplot(hues.df) +
		geom_rect(aes( xmin = get("theta") - 0.001, xmax = get("theta") + 0.001, color = get("colors"), fill = get("colors")),
							ymin = get("y.inner"), ymax = get("y.outer"), alpha = alpha) +
		coord_polar(theta = "x", start = -pi / 2, direction = -1, clip = "on") +
		scale_color_identity() +
		scale_fill_identity() +
		guides(fill = "none", color = "none") +
		theme_void() +
		ylim(c(0, ymax)) +
		geom_segment(aes(x = x1, y = y1, xend = x2, yend = y2),
								 data = data.frame(x1 = 0, x2 = 1.75 * pi, y1 = (y.outer + y.text) / 2, y2 = (y.outer + y.text) / 2),
								 inherit.aes = FALSE,
								 alpha = 0.5,
								 arrow = arrow(length = unit(0.05, "npc"))) +
		geom_text(data = hue_text.df, aes_string(x = "theta", y = "y.text", label = "label", hjust = "hjust"), size = text.size) +
		theme(plot.margin = unit(c(0, 0, 0, 0), "pt"))
	
	if (addStageLabel) {
		legend.p <- legend.p +
			annotate(geom = "text", x = G1.pos, y = (y.outer + y.inner) / 2, label = "G1/G0", size = text.size, color = "white") +
			annotate(geom = "text", x = S.pos, y = (y.outer + y.inner) / 2, label = "S", size = text.size, color = "black") +
			annotate(geom = "text", x = G2M.pos, y = (y.outer + y.inner) / 2, label = "G2|M", size = text.size, color = "black") 
	}
	
	
	
	return(legend.p)
}

#' Diagnostic function for UMI based datasets
#'
#' @description  The function will fit loess line for total UMIs numbers over
#' cell cycle position to diagnose non-fitting data, of which cells are not cycling.
#'
#'
#'
#' @param theta.v The cell cycle position - a numeric vector with range between 0 to 2pi.
#' @param totalumis The total UMIs number for each cell (without log2 transformation) - a numeric vector with the same length as \code{theta.v}.
#' @param span The parameter \eqn{\alpha} which controls the degree of smoothing. See \code{\link[stats]{loess}}. Default: 0.3
#' @param length.out The number of data points on the fitted lines to be output in the prediction data.frame. Default: 200
#' @param plot If \code{TRUE}, a \code{ggplot} scatter plot will be included in the output list. The figure will plot log2(totalumis) ~ theta.v 
#' with points and the fitted \code{\link[stats]{loess}} line. Default: TRUE
#' @param fig.title The title of the figure. Default: NULL
#' @param point.size  The size of the point in scatter plot used by \code{\link[scattermore]{geom_scattermore}}. Default: 2.1
#' @param point.alpha  The alpha value (transparency) of the point in scatter plot used by \code{\link[scattermore]{geom_scattermore}}. Default: 0.6
#' @param line.size  The size of the fitted line, used by \code{\link[ggplot2]{geom_path}}. Default: 0.8
#' @param line.alpha  The alpha value (transparency) of the fitted line, used by \code{\link[ggplot2]{geom_path}}. Default: 0.8
#' @param x_lab Title of x-axis. Default: "\eqn{\theta}"
#' @param y_lab Title of y-axis. Default: "log2(totalumis)"
#' @param ... Other arguments input to \code{\link[stats]{loess}}.
#'
#' @return A diagnostic message and a list with the following elements:
#' \itemize{
#'   \item fitted - The fitted vaues on the loess line. A vector of the length of y.
#'   \item residual - The residual values from the fitted loess line, i.e. y - y.fit. A vector of the length of y.
#'   \item pred.df - The prediction \code{data.frame} by uniformly sampling theta from 0 - 2*pi. Names of variables: \code{x} and \code{y}. The number of rows equals to \code{length.out}.
#'   \item loess.o - The fitted loess object.
#'   \item rsquared -  The coefficient of determination R2. Calculated as 1 - residual sum of squares / the total sum of squares.
#'   \item fig - When \code{plot} is \code{TRUE}, a \code{ggplot} scatter plot object will be returned with other items.
#' }
#'
#' @details This function fit a loess line between cell cycle position and 
#' log2 transformed total UMI number, as described in \code{\link{fit_periodic_loess}}.
#' If almost all cells are not cycling in a dataset, the estimated cell cycle positions
#' might be incorrect due to the shifted embedding center. Using the fact that the cell
#' should have highest total UMI number at the end of S phase and almost half of that 
#' highest total UMI number at M phase, we could detect those datasets which should 
#' be analysesd and intepreted carefully when using tricycle package. For such probelmatic
#' datasets, the defaul embedding center (0, 0) could lead to wrong inference. Thus, 
#' We don't rececommend using cell cycle position values if you get warnings from the 
#' \code{diagnose_totalumi} function.
#'
#'
#' @name diagnose_totalumi
#' @aliases diagnose_totalumi
#' @seealso
#' \code{\link{fit_periodic_loess}}.
#'
#' @author Shijie C. Zheng
#'
#' @examples
#' data(neurosphere_example, package = "tricycle")
#' neurosphere_example <- estimate_cycle_position(neurosphere_example)
#' diagnose.l <- diagnose_totalumi(neurosphere_example$tricyclePosition,
#'  neurosphere_example$TotalUMIs, plot = TRUE)
NULL

#' @export
#' 
diagnose_totalumi <- function(theta.v, totalumis, span = 0.3, length.out = 200, plot = TRUE,
															 fig.title = NULL, point.size = 2.1, point.alpha = 0.6,
															 line.size = 0.8, line.alpha = 0.8, x_lab = "\u03b8",
															 y_lab = "log2(totalumis)",  ...) {
	stopifnot("theta.v need to be between 0 - 2pi." = (min(theta.v) >= 0) & (max(theta.v) <= 2 * pi), 
						"The length of theta.v and totalumis should be the same." = length(theta.v) == length(totalumis))
	
	fit.l <- fit_periodic_loess(theta.v = theta.v, y = log2(totalumis + 1),
															span = span, length.out = length.out, plot = plot,
															fig.title = fig.title, point.size = point.size,
															point.alpha = point.alpha, line.size = line.size,
															line.alpha = line.alpha, color.vars = NULL,
															color.name = NULL, x_lab = x_lab, y_lab = y_lab,
															hue.colors = NULL, ... = ...)
	peak.v <- max(fit.l$pred.df$y[which((fit.l$pred.df$x > 0.75 * pi) & (fit.l$pred.df$x < 1.25 * pi))])
	valley.v <- min(fit.l$pred.df$y[which((fit.l$pred.df$x > 1.35 * pi) & (fit.l$pred.df$x < 1.85 * pi))])
	diff.v <- peak.v - valley.v
	if (diff.v < 0.4) {
		warning("The difference of predicted log2(total UMIs) between the end of S phase and M phase is less than 0.4.
						It seems that the predicted cell cycle position might not be correct due to the shift of embedding center. 
						This could be caused by that most cells in the dataset are not cycling. 
						We don't rececommend using cell cycle position values in this case.")
	} else {
		message(paste0("Message: The difference of predicted log2(total UMIs) between the end of S phase and M phase is ", sprintf("%.2f", diff.v), "(within tolerance)."))
	}
	
	if (mean((theta.v > 0.5 * pi )& (theta.v < 1.8 * pi)) > 0.5) {
		warning("The percentage of cell in positions between 0.5\u03C0 to 1.8\u03C0 is greater than 50%.
						It seems that the predicted cell cycle position might not be correct due to the shift of embedding center. 
						This could be caused by that most cells in the dataset are not cycling. 
						We don't rececommend using cell cycle position values in this case.")
	}
	
	return(fit.l)
}

#' Run PCA on Gene Ontology cell cycle genes
#'
#' Run PCA on Gene Ontology cell cycle genes abd get a new \linkS4class{SingleCellExperiment}.
#' User could use this function to learn new reference projection matrix.
#'
#' @param sce.o A \linkS4class{SingleCellExperiment} contains library size normalized **log-expression** matrix.
#' @param gname Alternative rownames of \code{sce.o}. If provided, this will be used to map genes within Gene Ontology cell cycle gene list.
#' If not provided, the rownames of \code{sce.o} will be used instead. Default: NULL
#' @param exprs_values Integer scalar or string indicating which assay of \code{sce.o} contains the **log-expression** values, which will be used to run PCA.
#' Default: 'logcounts'
#' @param gname.type The type of gene names as in \code{gname} or rownames of \code{sce.o}. It can be either 'ENSEMBL' or 'SYMBOL'. Default: 'ENSEMBL'
#' @param species The type of species in \code{sce.o}. It can be either 'mouse' or 'human'. If the user uses
#' custom \code{cycleGene.l}, this value will have no effect. Default: 'mouse'
#' @param AnnotationDb An AnnotationDb objects. It is used to map ENSEMBL IDs to gene SYMBOLs.
#'  If no AnnotationDb object being given, the function will use \code{\link[org.Hs.eg.db]{org.Hs.eg.db}} or \code{\link[org.Mm.eg.db]{org.Mm.eg.db}} for human and mouse respectively.
#' @param ntop The number of genes with highest variance to use when calculating PCA, as in \code{\link[scater]{calculatePCA}}. Default: 500
#' @param ncomponents The number of component components to obtain, as in \code{\link[scater]{calculatePCA}}. Default: 20
#' @param name String specifying the name to be used to store the result in the \code{\link[SingleCellExperiment]{reducedDims}} of the output. Default: 'PCA'
#' 
#' 
#' @details
#' The function require an output of a \linkS4class{SingleCellExperiment} object which contains the library size normalized **log-expression** matrix. The full dataset will
#' be subsetted to genes in the Gene Ontology cell cycle gene list (GO:0007049). The corresponding AnnotationDb object will be
#'  \code{\link[org.Mm.eg.db]{org.Mm.eg.db}} and \code{\link[org.Hs.eg.db]{org.Hs.eg.db}} for mouse and human respectively. If \code{runSeuratBy} is set, the data will be
#'  integrated to remove batch effect between samples/batches by Seurat.
#'
#' User can use this function to make new reference projection matrix by getting the 'rotation' attribute in PCA results. Such as
#' \code{attr(reducedDim(sce.o, 'PCA'), 'rotation')[, 1:2]}. See examples for more details.
#'
#' @return
#' A subset \linkS4class{SingleCellExperiment} object with only GO cell cycle genes will be return.
#' The PCA resulting will be save in reducedDims with chosen name \code{\link[SingleCellExperiment]{reducedDims}(..., name)}.
#' If Seurat integration is performed, another reducedDims with name 'matched.'+\code{name} will also be included in the \linkS4class{SingleCellExperiment}.
#'
#' @name run_pca_cc_genes
#'
#' @author Shijie C. Zheng
#'
#' @examples
#' data(neurosphere_example, package = "tricycle")
#' ### Use internal NeuroRef to project and infer tricyclePosition
#' neurosphere_example <- estimate_cycle_position(neurosphere_example) 
#' 
#' ### Build new reference
#' gocc_sce.o <- run_pca_cc_genes(neurosphere_example)
#' new.ref <- attr(reducedDim(gocc_sce.o, "PCA"), "rotation")[, seq_len(2)]
#' 
#' ### Use new reference to project and infer tricyclePosition
#' new_sce <- estimate_cycle_position(neurosphere_example, ref.m = new.ref,
#'  dimred = "tricycleEmbedding2")
#' plot(neurosphere_example$tricyclePosition, new_sce$tricyclePosition)
NULL



#' @importFrom AnnotationDbi select
#' @importFrom methods is
#' @importMethodsFrom scater runPCA
#' @importMethodsFrom SummarizedExperiment  assay colData assayNames
#' @importClassesFrom SingleCellExperiment SingleCellExperiment
#' @export
run_pca_cc_genes <- function(sce.o, gname = NULL, exprs_values = "logcounts", gname.type = c("ENSEMBL", "SYMBOL"), species = c("mouse", "human"), ntop = 500, ncomponents = 20,
      name = "PCA", AnnotationDb = NULL) {
    stopifnot("sce.o must be a SingleCellExperiment objet." = is(sce.o, "SingleCellExperiment"),
              "the designated assay does not exist in sce.o" = exprs_values %in% assayNames(sce.o))
    species <- match.arg(species)
    gname.type <- match.arg(gname.type)
    AnnotationDb <- .getAnnotationDB(AnnotationDb, species)

    if (is.null(gname)) {
        message("No gname input. Rownames of sce.o will be used.")
        gname <- rownames(sce.o)
    } else {
      stopifnot("gname does not match nrow sce.o!." = nrow(sce.o) == length(gname))
    }

    cycle.anno <- AnnotationDbi::select(AnnotationDb, keytype = "GOALL", keys = "GO:0007049", columns = gname.type)[, gname.type]
    gene.idx <- which(gname %in% cycle.anno)
    if (length(gene.idx) < 100) {
          stop("Less than 100 Gene Ontology cell cycle genes found in your data. Check you data or gname input.")
      }
    message(paste0(length(gene.idx), " out of ", length(cycle.anno), " Gene Ontology cell cycle genes found in your data."))
    GO.o <- sce.o[gene.idx, ]
    GO.o <- runPCA(GO.o, exprs_values = exprs_values, name = name, ntop = ntop, ncomponents = ncomponents)

    return(GO.o)
}

At least at this early stage in its life, `tricycle` is pretty much "oremanj's bikeshed
of interesting-but-maybe-not-fully-proven Trio ideas". PRs fixing bugs, extending
existing code to be more robust or complete, and/or improving documentation are very welcome,
but if you want to contribute a largish new feature, please raise it on the issue tracker
first so I have a chance to think about whether I want to take on the maintenance
burden or not.

We mostly follow the Trio contributing guide:
    https://trio.readthedocs.io/en/latest/contributing.html
but (this being largely a personal project) contributors will not automatically get commit
bits, and we're trying out some alternative tooling (code formatting with `black` and
enforced static type checking to pass `mypy --strict`) compared to the mainline Trio
projects. `tricycle` anticipates moving to live under the python-trio Github organization
once its scope has stabilized a bit, at which point it will stop being so much of a special
snowflake.

---
title: "tricycle: Transferable Representation and Inference of Cell Cycle"
author: 
- name: Shijie C. Zheng
  affiliation: Department of Biostatistics, Johns Hopkins Bloomberg School of Public Health
  email: shijieczheng@gmail.com
package: tricycle
output: 
  BiocStyle::html_document
bibliography: tricycle.bib
vignette: >
  %\VignetteIndexEntry{tricycle: Transferable Representation and Inference of Cell Cycle}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r, echo=FALSE}	
htmltools::img(src = knitr::image_uri(file.path("../man/figures", "logo.png")),
               alt = 'logo',
               style = 'position:absolute; top:50px; right:5px; padding:10px;height:200px')
```

# Introduction

Here we describe a package for inferring cell cycle position for a single-cell
RNA-seq dataset. The theoretical justification as well as benchmarks are
included in [@Zheng.2022]. In our hands, our approach (called TriCycle) works
robustly across a variety of data modalities including across species (human
and mouse), cell types and assay technology (10X, Fluidigm C1); we have yet to
encounter a dataset where this approach does not work. The main output is a
continuous estimate of the relative time within the cell cycle, represented as
a number between 0 and 2pi (which we refer to as cell cycle position). In
addition to the estimation process, we include a number of convenience functions
for visualizing cell cycle time and we also provide an implementation of a
discrete cell cycle stage predictor.
 
# Prerequisites

```{r setup, message = FALSE}
library(tricycle)
```
We recommend users to start with a *SingleCellExperiment* object. The output
will usually be the *SingleCellExperiment* with new info added. The functions
work on *matrix* or *SummarizedExperiment* objects although the output changes,
since these type of objects do not have the capability to store both the input
object and the estimates.

In the package, we include a example *SingleCellExperiment* dataset, which is a
real subset of mouse Neurosphere RNAseq of 2 samples. 200 cells from sample AX1
and AX2 were randonly sampled from the full data. This dataset is the same data
as we use for constructing our cell cycle embedding.
```{r example, message=FALSE}
data(neurosphere_example)
```

**Important**: Please note that the user should **normalize library size**
before putting into the tricycle functions. The library size normalization could
be done by *normalizeCounts* function in *scater* package or by calculating CPM
values.

# Overview of the package functionality

The method is based on taking a new dataset and projecting it into an embedding
representing cell cycle. This embedding is constructed using a reference
dataset. What is perhaps surprising is our finding that the same embedding is
appropriate for all the experiments we have looked at, including across species,
cell types and datasets. We are providing this embedding space as part of the
package, and we do not expect users to change this embedding (although the
functions in the package supports other embeddings).

The method is simple: you take a new dataset and project it into the latent
space and infer cell cycle time. The key functions here are

- `project_cycle_space()`
- `estimate_cycle_position()`

The next step is to verify that the cell cycle time was successfully predicted.
This involves looking at a number of useful plots. This involves a number of
useful visualization. Note for example our use of color scheme - because cell
cycle time "wraps around" the $[0,2\pi]$ interval, it is very useful to use a
color palette which also "wraps around". The relevant functions are

- `plot_emb_circle_space()`
- `circle_space_legend()`
- `fit_periodic_loess()`

We also provide a separate cell cycle stage predictor, predicting 5 different
stages; `estimate_Schwabe_stage()`. This predictor is a small modification of the
method proposed by [@Schwabe.2020]. __We include this function only for the purpose of convenience.__  
__This is not part of tricycle method!__

Finally we have a set of functions for creating your own reference latent space.


# Project a single cell data set to pre-learned cell cycle space

`project_cycle_space()` will automatically project the assay with name
`logcounts` into the cell cycle embedding without any other argument input. You
could specify species (default as mouse), gene IDs, gene ID type, and
`AnnotationDb` object if gene mapping is needed. Refer to
`man(project_cycle_space)` for details.

```{r project, message = TRUE}
neurosphere_example <- project_cycle_space(neurosphere_example)
neurosphere_example
```

The projected cell cycle space will be stored in *reducedDims* with name
"tricycleEmbedding" (you could set other embedding name.).
```{r plot_projection, message = FALSE}
library(ggplot2)
library(scattermore)
library(scater)
scater::plotReducedDim(neurosphere_example, dimred = "tricycleEmbedding") +
    labs(x = "Projected PC1", y = "Projected PC2") +
    ggtitle(sprintf("Projected cell cycle space (n=%d)",
                    ncol(neurosphere_example))) +
    theme_bw(base_size = 14)
```

# Infer cell cycle position

Once the new data has been projected into the cell cycle embedding, cell cycle
position is estimated using `estimate_cycle_position()`. If the data has not
been projected, this function will do the projection for you. Assuming a
*SingleCellExperiment* as input, the cell cycle position will be addded to the
`colData` of the object, with the name `tricyclePosition`.

```{r tricyclePosition, message = TRUE}
neurosphere_example <- estimate_cycle_position(neurosphere_example)
names(colData(neurosphere_example))
```

The estimated cell cycle position is bound between 0 and 2pi. Note that we
strive to get high resolution of cell cycle state, and we think the continuous
position is more appropriate when describing the cell cycle. However, to help
users understand the position variable, we also note that users can
approximately relate 0.5pi to be the start of S stage, pi to be the start of G2M
stage, 1.5pi to be the middle of M stage, and 1.75pi-0.25pi to be G1/G0 stage.

# Assessing performance

We have two ways of (quickly) assessing whether TriCycle works. They are

1. Look at the projection of the data into the cell cycle embedding.
2. Look at the expression of key genes as a function of cell cycle position.

Plotting the projection of the data into the cell cycle embedding is shown
above. Our observation is that deeper sequenced data will have a more clearly
ellipsoid pattern with an empty interior. As sequencing depth decreases, the
radius of the ellipsoid decreases until the empty interior disappears. So the
absence of an interior does not mean the method does not work.

It is more important to inspect a couple of genes as a function of cell cycle
position. We tend to use Top2a which is highly expressed and therefore
"plottable" in every dataset. Other candidates are for example Smc2. To plot
this data, we provide a convenient function `fit_periodic_loess()` to fit a
loess line between the cyclic variable $\theta$ and other response variables.
This fitting is done by making `theta.v` 3 periods
`(c(theta.v - 2 * pi, theta.v, theta.v + 2 * pi))` and repeating `y` 3 times.
Only the fitted values corresponding to original `theta.v` will be returned.
In this example, we show how well the expression of the cell cycle marker gene
*Top2a* change along $\theta$.
```{r loess, message = TRUE}
top2a.idx <- which(rowData(neurosphere_example)$Gene == 'Top2a')
fit.l <- fit_periodic_loess(neurosphere_example$tricyclePosition,
                            assay(neurosphere_example, 'logcounts')[top2a.idx,],
                            plot = TRUE,
                       x_lab = "Cell cycle position \u03b8", y_lab = "log2(Top2a)",
                       fig.title = paste0("Expression of Top2a along \u03b8 (n=",
                                          ncol(neurosphere_example), ")"))
names(fit.l)
fit.l$fig + theme_bw(base_size = 14)
```

For Top2a we expect peak expression around $\pi$.

# Alternative: Infer cell cycle stages

This method was proposed by @Schwabe.2020. We did small modifications to
reduce `NA` assignments. But on average, the performance is quite similar to the
original implementation in [Revelio](https://github.com/danielschw188/Revelio/)
package. In brief, we calculate the *z*-scores of highly expressed stage
specific cell cycle marker genes, and assgin the cell to the stage with the
greatest *z*-score.
```{r stage, message = TRUE}
neurosphere_example <- estimate_Schwabe_stage(neurosphere_example,
                                            gname.type = 'ENSEMBL',
                                            species = 'mouse')
scater::plotReducedDim(neurosphere_example, dimred = "tricycleEmbedding",
                       colour_by = "CCStage") +
  labs(x = "Projected PC1", y = "Projected PC2",
       title = paste0("Projected cell cycle space (n=", ncol(neurosphere_example), ")")) +
  theme_bw(base_size = 14)
```


# Plot out the kernel density

Another useful function is *plot_ccposition_den*, which computes kernel density
of $\theta$ conditioned on a phenotype using von Mises distribution. The ouput
figures are provided in two flavors, polar coordinates and Cartesian
coordinates. This could be useful when comparing different cell types,
treatments, or just stages. (Because we use a very small dataset here as
example, we set the bandwith, i.e. the concentration parameter of the von Mises
distribution as 10 to get a smooth line.)

```{r density, message = TRUE}
plot_ccposition_den(neurosphere_example$tricyclePosition,
                    neurosphere_example$sample, 'sample',
                    bw = 10, fig.title = "Kernel density of \u03b8") +
  theme_bw(base_size = 14)

plot_ccposition_den(neurosphere_example$tricyclePosition,
                    neurosphere_example$sample, 'sample', type = "circular",
                    bw = 10,  fig.title = "Kernel density of \u03b8") +
  theme_bw(base_size = 14)
```


# Plot out embedding scater plot colored by cell cycle position
To visualize the cell cycle position $\theta$ on any embedding, we need to
carefully choose a cyclic color palette. Thus, we include such functions to
plot any embedding of *SingleCellExperiment* object with cyclic variables. A
companion helper function to create the cyclic legend is also available.

```{r cyclic, message = TRUE, fig.width = 10, fig.height = 7}
library(cowplot)
p <- plot_emb_circle_scale(neurosphere_example, dimred = 1,
                           point.size = 3.5, point.alpha = 0.9) +
  theme_bw(base_size = 14)
legend <- circle_scale_legend(text.size = 5, alpha = 0.9)
plot_grid(p, legend, ncol = 2, rel_widths = c(1, 0.4))
```

We plot our our projection embedding. In practice, user could use other
embedding, such as UMAP or t-SNE and get informative representations too.

# Make a new reference

Users could make their own reference by doing PCA on the cell cycle genes, and
use the learned rotation matrix as the reference matrix in other functions. Here
is an example, we just use *run_pca_cc_genes* function to extract Gene Ontology
cell cycle genes (GO:0007049) and run PCA. By projecting the data itself with
the learned reference, the projections are equivalent to direct PCA results.
But you could use this newly learned reference to project other datasets.

```{r newRef, message = TRUE}
set.seed(100)
gocc_sce.o <- run_pca_cc_genes(neurosphere_example,
                               exprs_values = "logcounts", species = "mouse")
new.ref <- attr(reducedDim(gocc_sce.o, 'PCA'), 'rotation')[, seq_len(2)]
head(new.ref)
new_sce <- estimate_cycle_position(neurosphere_example, ref.m  = new.ref,
                                   dimred = 'tricycleEmbedding2')

```

Note: If user wants to calculate correlation between two cyclic variables, such
as cell cycle position, traditional pearson's correlation coefficient won't
consider the cyclic nature. Users could use (absolute) circular correlation
values instead. (The signs of PC1 and PC2 are not deterministic when
re-learning the reference by performing PCA. If the PC1 is flipped, there will
be a $\pi$ shift. So does PC2. If the user fixes the reference, there won't be
any flipping. But considering the variations around $0$ or $2\pi$, circular
correlation should still be used instead of pearson's correlation coefficient.)
```{r cor, message = TRUE}
cor(neurosphere_example$tricyclePosition, new_sce$tricyclePosition)
CircStats::circ.cor(neurosphere_example$tricyclePosition, new_sce$tricyclePosition)
qplot(x = neurosphere_example$tricyclePosition,y = new_sce$tricyclePosition) +
  labs(x = "Original \u03b8", y = "New \u03b8",
       title = paste0("Comparison of two \u03b8 (n=", ncol(neurosphere_example), ")")) +
  theme_bw(base_size = 14)
```

# Make a new reference using datasets with batch effects
This section introduce how to make a new reference using dataset with batch
effects. It is only recommended for expert users who identifies batch effects in
their data and want to use that data to build a custom reference.
In theory, the users could use other methods to remove batch effect. Here, we
use *Seurat*, which is used to construct our Neurosphere reference
(@Zheng.2022), as an example. (The code in this section is not evaluated.)
```{r batch, eval = FALSE, echo = TRUE}
# suppose we have a count matrix containing all cells across batches; we first subset the matrix to GO cell cycle genes
library(org.Mm.eg.db)
library(AnnotationDbi)
cc.genes <- AnnotationDbi::select(org.Mm.eg.db, keytype = "GOALL",
                                  keys = "GO:0007049",
                                  columns = "ENSEMBL")[, "ENSEMBL"]
count_cc.m <- count.m[ensembl.ids %in% cc.genes, ]  # ensembl.ids is the ensembl.ids for each row of count.m

# we then construct a Seurat object using the subset matrix and set the batch variable
library(Seurat)
seurat.o <- CreateSeuratObject(counts = count_cc.m)
seurat.o[["batch"]] <- batch.v

# make a Seurat list and normalize for each batch separately
# variable features definition is required for FindIntegrationAnchors function
seurat.list <- lapply(SplitObject(seurat.o, split.by = "batch"),
                      function(x) FindVariableFeatures(NormalizeData(x)))

# find anchors and merge data
seurat.anchors <- FindIntegrationAnchors(object.list = seurat.list)
seurat.integrated <- IntegrateData(anchorset = seurat.anchors)
corrected.m <- seurat.integrated@assays$integrated@data

# run PCA on the batch effects corrected matrix and get the rotaions scores for the top 2 PCs
pca.m <- scater::calculatePCA(corrected.m, ntop = 500)
new.ref <- attr(pca.m, 'rotation')[, seq_len(2)]

```


# Session info {.unnumbered}

```{r sessionInfo, echo=FALSE}
sessionInfo()
```

# References





In the package, we provide a reference, learned from the full dataset of the
mouse Neurosphere RNAseq. The reference gives weights of 500 cell cycle genes
and their IDs. Although learned from mouse, it is applicable to human data as
well, with the gene mapped by gene symbols.
```{r ref, message = FALSE}
data(neuroRef)
head(neuroRef)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/project_cycle_space.R
\name{project_cycle_space}
\alias{project_cycle_space}
\title{Project data into the cell cycle pattern space}
\usage{
project_cycle_space(
  x,
  exprs_values = "logcounts",
  altexp = NULL,
  name = "tricycleEmbedding",
  ref.m = NULL,
  gname = NULL,
  gname.type = c("ENSEMBL", "SYMBOL"),
  species = c("mouse", "human"),
  AnnotationDb = NULL
)
}
\arguments{
\item{x}{A numeric matrix of **log-expression** values where rows are features and columns are cells.
Alternatively, a \linkS4class{SummarizedExperiment} or \linkS4class{SingleCellExperiment} containing such a matrix.}

\item{exprs_values}{Integer scalar or string indicating which assay of \code{x} contains the **log-expression** values. Default: 'logcounts'}

\item{altexp}{String or integer scalar specifying an alternative experiment containing the input data.}

\item{name}{String specifying the name to be used to store the result in the \code{\link[SingleCellExperiment]{reducedDims}} of the output. Default: 'tricycleEmbedding'}

\item{ref.m}{A custom reference projection matrix to project the new data, where rows are features and columns are dimensions.
Users need to use the same type of \code{gname}(or rownames of \code{x}) as for the \code{ref.m}.
If no custom ref.m is given, the internal reference \code{neuroRef} will be used.}

\item{gname}{Alternative rownames of \code{x}. If provided, this will be used to map genes within \code{x} with genes in \code{ref.m}.
If not provided, the rownames of \code{x} will be used instead. Default: NULL}

\item{gname.type}{The type of gene names as in \code{gname} or rownames of \code{x}. It can be either 'ENSEMBL' or 'SYMBOL'. If the user uses
custom \code{ref.m}, this value will have no effect. Default: 'ENSEMBL'}

\item{species}{The type of species in \code{x}. It can be either 'mouse' or 'human'. If the user uses
custom \code{ref.m}, this value will have no effect. Default: 'mouse'}

\item{AnnotationDb}{An AnnotationDb objects. If the user uses the internal reference to project human data,
and provide rownames in the format of Ensembl IDs, this object will be used to map Ensembl IDs to gene SYMBOLs.
If no AnnotationDb object being given, the function will use \code{\link[org.Hs.eg.db]{org.Hs.eg.db}}.}
}
\value{
If the input is a numeric matrix or a \linkS4class{SummarizedExperiment}, a projection matrix with rows cells and column dimensions will be returned.
The actual rotation matrix used to project the data is included in the attributes with name 'rotation'.

For \linkS4class{SingleCellExperiment}, an updated \linkS4class{SingleCellExperiment} is returned containing projection matrix in \code{\link[SingleCellExperiment]{reducedDims}(..., name)}.
}
\description{
Project mouse and human single cell RNAseq data into a cell cycle embedding by a pre-learned reference projection matrix.
}
\details{
The function will use pre-learned cell cycle pattern to project new data to show the cell cycle progression. If the user uses internal Neuropshere reference,
the expression values must be **log-transformed**. Besides, we would assume the input data has been already preprocessed, library size normalized at least.
The projection process is to take sum of weighted mean-centered expression of chosen genes, so the mean expression of a given gene could be affected without library size normalization.
}
\examples{
data(neurosphere_example, package = "tricycle")
neurosphere_example <- project_cycle_space(neurosphere_example)
reducedDimNames(neurosphere_example)
head(reducedDim(neurosphere_example, "tricycleEmbedding"))
plot(reducedDim(neurosphere_example, "tricycleEmbedding"))
names(attributes(reducedDim(neurosphere_example, "tricycleEmbedding")))
}
\references{
Zheng SC, et al.
\emph{Universal prediction of cell cycle position using transfer learning.}
Genome Biology (2022) 23: 41
doi:\href{https://doi.org/10.1186/s13059-021-02581-y}{10.1186/s13059-021-02581-y}.
}
\seealso{
\code{\link{estimate_cycle_position}}, for inferring cell cycle position.
}
\author{
Shijie C. Zheng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_emb_circle_scale.R
\name{plot_emb_circle_scale}
\alias{plot_emb_circle_scale}
\title{Plot embedding with cyclic cell cycle position}
\usage{
plot_emb_circle_scale(
  sce.o,
  color_by = "tricyclePosition",
  facet_by = NULL,
  dimred = 1,
  dim = seq_len(2),
  fig.title = NULL,
  point.size = 2.1,
  point.alpha = 0.6,
  x_lab = NULL,
  y_lab = NULL,
  hue.colors = c("#2E22EA", "#9E3DFB", "#F86BE2", "#FCCE7B", "#C4E416", "#4BBA0F",
    "#447D87", "#2C24E9"),
  hue.n = 500,
  plot.legend = FALSE
)
}
\arguments{
\item{sce.o}{A \linkS4class{SingleCellExperiment} contains the embbing to be plotted against.}

\item{color_by}{The name of variable in \code{colData(sce.o)} to be used to show colors. Default: "tricyclePosition"}

\item{facet_by}{The name of variable in \code{colData(sce.o)} to be used to facet scatter plots. If used, the function will return a list of ggplot objects. If NULL, no faceted panels will be returned. Default: NULL}

\item{dimred}{The name or index of reducedDims in  \linkS4class{SingleCellExperiment} (\code{\link[SingleCellExperiment]{reducedDims}}). Default: 1}

\item{dim}{The indices of \code{dimred} to be plotted. At the moment, it has to be two integers.   Default: 1:2}

\item{fig.title}{The title of the figure. Default: NULL}

\item{point.size}{The size of the point in scatter plot used by \code{\link[scattermore]{geom_scattermore}}. Default: 2.1}

\item{point.alpha}{The alpha value (transparency) of the point in scatter plot used by \code{\link[scattermore]{geom_scattermore}}. Default: 0.6}

\item{x_lab}{Title of x-axis. If not given, the colname of \code{dimred} will be used. Default: NULL}

\item{y_lab}{Title of y-axis. If not given, the colname of \code{dimred} will be used. Default: NULL}

\item{hue.colors}{The string vector gives the cyclic colors. The first color should look very similar to the last one.
Default: c("#2E22EA", "#9E3DFB", "#F86BE2", "#FCCE7B", "#C4E416", "#4BBA0F", "#447D87", "#2C24E9")}

\item{hue.n}{The number of breaks of color scheme. Default: 500}

\item{plot.legend}{Whether the legend should be plotted with the scatter plot. We recommend not to use this legend but use the cyclic legend produced by \code{\link{circle_scale_legend}} instead. Default: FALSE}
}
\value{
A ggplot object or a list of ggplot objects.
If \code{facet_by} is not assigned, a single ggplot plot of the scatter plot will be return,
Otherwise, apart from the first scatter plot showing all cells together, other faceted scatter plots will also be given in the list.
}
\description{
Generate scat plot of embedding with cyclic cell cycle position or other cyclic variables
}
\details{
This function help users plot embedding scater plot colored by cyclic variables, such as cell cycle position, which is bound between 0 - 2pi.
It will take a \linkS4class{SingleCellExperiment} object as input, and plot out its \code{dimred} such as PCA, UMAP, and etc with a cyclic color scheme.
}
\examples{
data(neurosphere_example, package = "tricycle")
neurosphere_example <- estimate_cycle_position(neurosphere_example)
plot_emb_circle_scale(neurosphere_example, point.size = 3.1, point.alpha = 0.8)
}
\author{
Shijie C. Zheng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_periodic_loess.R
\name{fit_periodic_loess}
\alias{fit_periodic_loess}
\title{Fit periodic loess line with circular predictor}
\arguments{
\item{theta.v}{The cell cycle position - a numeric vector with range between 0 to 2pi.}

\item{y}{The response variable - a numeric vector with the same length as \code{theta.v}.}

\item{span}{The parameter \eqn{\alpha} which controls the degree of smoothing. See \code{\link[stats]{loess}}. Default: 0.3}

\item{length.out}{The number of data points on the fitted lines to be output in the prediction data.frame. Default: 200}

\item{plot}{If \code{TRUE}, a \code{ggplot} scatter plot will be included in the output list. The figure will plot y ~ theta.v 
with points and the fitted \code{\link[stats]{loess}} line. Default: FALSE}

\item{fig.title}{The title of the figure. Default: NULL}

\item{point.size}{The size of the point in scatter plot used by \code{\link[scattermore]{geom_scattermore}}. Default: 2.1}

\item{point.alpha}{The alpha value (transparency) of the point in scatter plot used by \code{\link[scattermore]{geom_scattermore}}. Default: 0.6}

\item{line.size}{The size of the fitted line, used by \code{\link[ggplot2]{geom_path}}. Default: 0.8}

\item{line.alpha}{The alpha value (transparency) of the fitted line, used by \code{\link[ggplot2]{geom_path}}. Default: 0.8}

\item{color.vars}{Optional. A vector of categorical variable of the same length of \code{theta.v}, and it will be used to color points in figure. Default: NULL}

\item{color.name}{The name of the color variables. Used as the name for legend. Default: NULL}

\item{x_lab}{Title of x-axis. Default: "\eqn{\theta}"}

\item{y_lab}{Title of y-axis. Default: "y"}

\item{hue.colors}{The string vector gives custom colors. If not given, the default \code{\link[ggplot2]{scale_color_discrete}} will be used. Default: NULL}

\item{...}{Other arguments input to \code{\link[stats]{loess}}.}
}
\value{
A list with the following elements:
\itemize{
  \item fitted - The fitted vaues on the loess line. A vector of the length of y.
  \item residual - The residual values from the fitted loess line, i.e. y - y.fit. A vector of the length of y.
  \item pred.df - The prediction \code{data.frame} by uniformly sampling theta from 0 - 2*pi. Names of variables: \code{x} and \code{y}. The number of rows equals to \code{length.out}.
  \item loess.o - The fitted loess object.
  \item rsquared -  The coefficient of determination R2. Calculated as 1 - residual sum of squares / the total sum of squares.
  \item fig - When \code{plot} is \code{TRUE}, a \code{ggplot} scatter plot object will be returned with other items.
}
}
\description{
The function will fit a loess line using cell cycle position and other variables, such as expression levels of a gene
 or log-transformed totalUMIs numbers. The circular nature of cell cycle position is taken into account by making 3 copies inside the function.
 For convenience, the function will also return a scatter plot with fitted line if needed.
}
\details{
This function fit a normal loess line, but take the circularity of cell cycle position into account by making \code{theta.v} 3 periods
(\code{c(theta.v - 2 * pi, theta.v, theta.v + 2 * pi)}) and repeating y 3 times. Only the fitted values corresponding to original \code{theta.v}
will be returned. For convenience, the function will also return a scatter plot with fitted line if needed. 
Or user can use \code{pred.df} to visualize the loess line themselves.
}
\examples{
data(neurosphere_example, package = "tricycle")
neurosphere_example <- estimate_cycle_position(neurosphere_example)
top2a.idx <- which(rowData(neurosphere_example)$Gene == "Top2a")
fit.l <- fit_periodic_loess(neurosphere_example$tricyclePosition,
 assay(neurosphere_example, "logcounts")[top2a.idx, ], plot = TRUE)
fit.l$fig
}
\seealso{
\code{\link{estimate_cycle_position}}, for inferring cell cycle position.
}
\author{
Shijie C. Zheng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/neuroRef-data.R
\docType{data}
\name{neuroRef}
\alias{neuroRef}
\title{Pre-learned reference projection matrix from the Neurosphere dataset}
\format{
An object of class `'data.frame'`, with 5 variables. Normally, user won't call this
data directly as it will be automatically used in \code{\link{project_cycle_space}} if no custom reference
projection matrix is provided. Each row is gene, and rotation scores for PC1 and PC2, mouse ENSEMBL IDs,
and mouse gene SYMBOLs are included. The `SYMBOL` values are just the upper-case `symbol` values.
}
\usage{
data(neuroRef)
}
\description{
Default reference projection matrix learned from the Neurosphere dataset.
}
\examples{
data(neuroRef)
}
\references{
Zheng SC, et al.
\emph{Universal prediction of cell cycle position using transfer learning.}
Genome Biology (2022) 23: 41
doi:\href{https://doi.org/10.1186/s13059-021-02581-y}{10.1186/s13059-021-02581-y}.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_emb_circle_scale.R
\name{circle_scale_legend}
\alias{circle_scale_legend}
\title{Get the cyclic legend}
\usage{
circle_scale_legend(hue.colors = c("#2E22EA", "#9E3DFB", "#F86BE2", "#FCCE7B", "#C4E416", "#4BBA0F", "#447D87", "#2C24E9"),
 hue.n = 500, alpha = 0.6, y.inner = 1.5, y.outer = 3, y.text = 3.8, ymax = 4.5, text.size = 3, addStageLabel = FALSE, 
 G1.pos = 0, S.pos = 2.2, G2M.pos = 3.9)
}
\arguments{
\item{hue.colors}{The string vector gives the cyclic colors. The first color should look very similar to the last one.
Default: c("#2E22EA", "#9E3DFB", "#F86BE2", "#FCCE7B", "#C4E416", "#4BBA0F", "#447D87", "#2C24E9")}

\item{hue.n}{The number of breaks of color scheme. Default: 500}

\item{alpha}{The alpha value (transparency). Default: 0.6}

\item{y.inner}{The radius of inner circle of the donut. Default: 1.5}

\item{y.outer}{The radius of outer circle of the donut. Default: 3}

\item{y.text}{The radius of text position. Default: 3.8}

\item{ymax}{The value control the border of the legend. Default: 4.5}

\item{text.size}{The size of the text Default: 3}

\item{addStageLabel}{Whether to add approximate discrete stage labels. Default: FALSE}

\item{G1.pos}{Approximate radius value of G1 label position. Default: 0}

\item{S.pos}{Approximate radius value of S label position. Default: 2.2}

\item{G2M.pos}{Approximate radius value of G2M label position. Default: 3.9}
}
\value{
A ggplot object
}
\description{
This function is a helper function to create the cyclic ggplot color legend.
}
\details{
The function will make a donut shape to serve as the cyclic color legend. The arguments should match the argument used in \code{\link{plot_emb_circle_scale}}.
}
\examples{
circle_scale_legend()
}
\author{
Shijie C. Zheng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/estimate_Schwabe_stage.R
\name{estimate_Schwabe_stage}
\alias{estimate_Schwabe_stage}
\title{Assign cell cycle stages using Schwabe method}
\usage{
estimate_Schwabe_stage(
  x,
  exprs_values = "logcounts",
  batch.v = NULL,
  altexp = NULL,
  cycleGene.l = NULL,
  gname = NULL,
  gname.type = c("ENSEMBL", "SYMBOL"),
  species = c("mouse", "human"),
  AnnotationDb = NULL,
  corThres = 0.2,
  tolerance = 0.3
)
}
\arguments{
\item{x}{A numeric matrix of **log-expression** values where rows are features and columns are cells.
Alternatively, a \linkS4class{SummarizedExperiment} or \linkS4class{SingleCellExperiment} containing such a matrix.}

\item{exprs_values}{Integer scalar or string indicating which assay of \code{x} contains the **log-expression** values, which will be used for projection.
If the projection already exists, you can ignore this value. Default: 'logcounts'}

\item{batch.v}{A string specifies which column in colData of \linkS4class{SummarizedExperiment} or \linkS4class{SingleCellExperiment} to use as the batch variable.
Or it can be a vector, of which the number of elements equals to the number of columns of \code{x}. The 5 stage cell cycle assignments are preformed for each batch separately.
No \code{NA} is permitted. Default: NULL}

\item{altexp}{String or integer scalar specifying an alternative experiment containing the **log-expression** data, which will be used for projection.
If the projection is already calculated and stored in the \linkS4class{SingleCellExperiment} as a dimred, leave this value to default NULL.}

\item{cycleGene.l}{A list contains the marker genes for each stage. The stage names should be included as names of the elements. If user feed custom list,
they should make sure that the same gene id type for \code{x} and \code{cycleGene.l}. If not custom list is given, \code{\link{RevelioGeneList}} will be used.
Default: NULL}

\item{gname}{Alternative rownames of \code{x}. If provided, this will be used to map genes within \code{x} with genes in \code{ref.m}.
If not provided, the rownames of \code{x} will be used instead. Default: NULL}

\item{gname.type}{The type of gene names as in \code{gname} or rownames of \code{x}. It can be either 'ENSEMBL' or 'SYMBOL'. If the user uses
custom \code{ref.m}, this value will have no effect. Default: 'ENSEMBL'}

\item{species}{The type of species in \code{x}. It can be either 'mouse' or 'human'. If the user uses
custom \code{cycleGene.l}, this value will have no effect. Default: 'mouse'}

\item{AnnotationDb}{An AnnotationDb objects. It is used to map ENSEMBL IDs to gene SYMBOLs.
If no AnnotationDb object being given, the function will use \code{\link[org.Hs.eg.db]{org.Hs.eg.db}} or \code{\link[org.Mm.eg.db]{org.Mm.eg.db}} for human and mouse respectively.}

\item{corThres}{For each batch and each stage, correlations between expression of each gene and the mean of all genes belonging to that stage
will be calculated to filter the final gene list used for inference. The genes with a correlation between \code{corThres} will not be used for calculating \emph{z}-scores.
Default: 0.2}

\item{tolerance}{For each cell, the function will compare the largest two \emph{z}-scores. If the difference between those two \emph{z}-scores is less than \code{tolerance},
the cell will be treated un-assignable with \code{NA} value returned for that cell. Default: 0.3}
}
\value{
If the input is a numeric matrix, the discretized cell cycle stages - a factor vector corresponding to each cell will be returned.

If the input is \linkS4class{SummarizedExperiment}, the original \linkS4class{SummarizedExperiment} with the discretized cell cycle stages stored in colData with name 'CCStage' will be returned.

If the input is \linkS4class{SingleCellExperiment}, the original \linkS4class{SingleCellExperiment} with the discretized cell cycle stages stored in colData with name 'CCStage' will be returned.
}
\description{
The function is a re-implementation of cell cycle stage assignment method
proposed in Schwabe et al.(2020), with a little modification. The core
 assignment method is not designed by the authors of this package!
}
\details{
The function is a re-implementation of cell cycle stage assignment method
proposed in Schwabe et al.(2020), with a little modification. We include 
this function only for the purpose of convenience. The core assignment method
is not designed by the authors of this package!
Breiefly, the function assigns cells to discretized cell cycle stages by
 comparing the \emph{z}-scores calculated for each stage markers.
Without cycleGene.l input, \code{\link{RevelioGeneList}} will be used.
If you use this function, you should cite Schwabe et al.(2020).
}
\examples{
data(neurosphere_example, package = "tricycle")
neurosphere_example <- estimate_Schwabe_stage(neurosphere_example,
 gname.type = "ENSEMBL", species = "mouse")
neurosphere_example2 <- estimate_Schwabe_stage(neurosphere_example, batch.v = "sample")
neurosphere_example3 <- estimate_Schwabe_stage(neurosphere_example,
 batch.v = neurosphere_example$sample)
neurosphere_example <- project_cycle_space(neurosphere_example)
plot(reducedDim(neurosphere_example, "tricycleEmbedding"),
 col = neurosphere_example$CCStage)
}
\references{
Schwabe D, et al.
\emph{The transcriptome dynamics of single cells during the cell cycle.}
Molecular Systems Biology (2020) 16: e9946
doi:\href{https://doi.org/10.15252/msb.20209946}{10.15252/msb.20209946}.

Zheng SC, et al.
\emph{Universal prediction of cell cycle position using transfer learning.}
Genome Biology (2022) 23: 41
doi:\href{https://doi.org/10.1186/s13059-021-02581-y}{10.1186/s13059-021-02581-y}.
}
\author{
Shijie C. Zheng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnose_totalumi.R
\name{diagnose_totalumi}
\alias{diagnose_totalumi}
\title{Diagnostic function for UMI based datasets}
\arguments{
\item{theta.v}{The cell cycle position - a numeric vector with range between 0 to 2pi.}

\item{totalumis}{The total UMIs number for each cell (without log2 transformation) - a numeric vector with the same length as \code{theta.v}.}

\item{span}{The parameter \eqn{\alpha} which controls the degree of smoothing. See \code{\link[stats]{loess}}. Default: 0.3}

\item{length.out}{The number of data points on the fitted lines to be output in the prediction data.frame. Default: 200}

\item{plot}{If \code{TRUE}, a \code{ggplot} scatter plot will be included in the output list. The figure will plot log2(totalumis) ~ theta.v 
with points and the fitted \code{\link[stats]{loess}} line. Default: TRUE}

\item{fig.title}{The title of the figure. Default: NULL}

\item{point.size}{The size of the point in scatter plot used by \code{\link[scattermore]{geom_scattermore}}. Default: 2.1}

\item{point.alpha}{The alpha value (transparency) of the point in scatter plot used by \code{\link[scattermore]{geom_scattermore}}. Default: 0.6}

\item{line.size}{The size of the fitted line, used by \code{\link[ggplot2]{geom_path}}. Default: 0.8}

\item{line.alpha}{The alpha value (transparency) of the fitted line, used by \code{\link[ggplot2]{geom_path}}. Default: 0.8}

\item{x_lab}{Title of x-axis. Default: "\eqn{\theta}"}

\item{y_lab}{Title of y-axis. Default: "log2(totalumis)"}

\item{...}{Other arguments input to \code{\link[stats]{loess}}.}
}
\value{
A diagnostic message and a list with the following elements:
\itemize{
  \item fitted - The fitted vaues on the loess line. A vector of the length of y.
  \item residual - The residual values from the fitted loess line, i.e. y - y.fit. A vector of the length of y.
  \item pred.df - The prediction \code{data.frame} by uniformly sampling theta from 0 - 2*pi. Names of variables: \code{x} and \code{y}. The number of rows equals to \code{length.out}.
  \item loess.o - The fitted loess object.
  \item rsquared -  The coefficient of determination R2. Calculated as 1 - residual sum of squares / the total sum of squares.
  \item fig - When \code{plot} is \code{TRUE}, a \code{ggplot} scatter plot object will be returned with other items.
}
}
\description{
The function will fit loess line for total UMIs numbers over
cell cycle position to diagnose non-fitting data, of which cells are not cycling.
}
\details{
This function fit a loess line between cell cycle position and 
log2 transformed total UMI number, as described in \code{\link{fit_periodic_loess}}.
If almost all cells are not cycling in a dataset, the estimated cell cycle positions
might be incorrect due to the shifted embedding center. Using the fact that the cell
should have highest total UMI number at the end of S phase and almost half of that 
highest total UMI number at M phase, we could detect those datasets which should 
be analysesd and intepreted carefully when using tricycle package. For such probelmatic
datasets, the defaul embedding center (0, 0) could lead to wrong inference. Thus, 
We don't rececommend using cell cycle position values if you get warnings from the 
\code{diagnose_totalumi} function.
}
\examples{
data(neurosphere_example, package = "tricycle")
neurosphere_example <- estimate_cycle_position(neurosphere_example)
diagnose.l <- diagnose_totalumi(neurosphere_example$tricyclePosition,
 neurosphere_example$TotalUMIs, plot = TRUE)
}
\seealso{
\code{\link{fit_periodic_loess}}.
}
\author{
Shijie C. Zheng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_ccposition_den.R
\name{plot_ccposition_den}
\alias{plot_ccposition_den}
\title{Plot cell cycle position kernel density stratified by a factor}
\arguments{
\item{theta.v}{The cell cycle position - a numeric vector with range between 0 to 2pi.}

\item{color_var.v}{A factor variable to stratify \code{theta.v}, such as samples or 'CCStage'. The length of it should equal to the length of \code{theta.v}.}

\item{color_name}{The name of the color_var.v to be used in the legend.}

\item{palette.v}{A string vector to give the color names. It should has the length of the number of levels of \code{color_var.v}. If not given, the 'Set1' palette will be used.
(See \code{\link[RColorBrewer]{display.brewer.all}}) If the number of levels
of \code{color_var.v} is greater than 8, only the top 8 levels of most cell
 will be shown. You can show them all by feeding enough colors in \code{palette.v}. Default: NULL}

\item{fig.title}{The title of the figure. Default: NULL}

\item{type}{It can be either of 'linear' or 'circular'. 'linear' corresponds to Cartesian coordinate system and 'circular' for polar coordinate system. Default: 'linear'}

\item{bw}{The smoothing bandwidth to be used. It is equal to the concentration parameter of the von Mises distribution. See \code{\link[circular]{density.circular}}. Default: 30}

\item{weighted}{Whether the density should be weighted on the percentage of each level of \code{color_var.v}. Default: FALSE}

\item{line.size}{The size of the line used by \code{\link[ggplot2]{geom_path}}. Default: 0.5}

\item{line.alpha}{The alpha value of the line used by \code{\link[ggplot2]{geom_path}}. Default: 0.5}

\item{addRug}{Whether to add rug on the bottom of the linear density plot or an inner circle on the circular plot to show the continuous scale of theta. Default: TRUE}

\item{RugPalette.v}{The palette used for the rug plot. If not given, it will used the same default palette as in \code{\link{plot_emb_circle_scale}}.}

\item{...}{Other arguments accepted by \code{\link[ggplot2]{geom_path}}.}
}
\value{
A ggplot object
}
\description{
The function will compute and plot cell cycle position kernel density.
}
\details{
The function first estimates kernel density using the von Mises distribution. Then,
it plots out the density in the polar coordinate system or Cartesian coordinate system. Different colors
represents different levels of \code{color_var.v} and the dashed black line is the marginal distribution of all cells.
}
\examples{
data(neurosphere_example, package = "tricycle")
neurosphere_example <- estimate_cycle_position(neurosphere_example)
plot_ccposition_den(neurosphere_example$tricyclePosition, neurosphere_example$sample, "sample")

neurosphere_example <- estimate_Schwabe_stage(neurosphere_example,
 gname.type = "ENSEMBL", species = "mouse")
plot_ccposition_den(neurosphere_example$tricyclePosition, neurosphere_example$CCStage, "CCStage")
}
\seealso{
\code{\link{estimate_Schwabe_stage}}, for inferring 5 stages of cell cycle
}
\author{
Shijie C. Zheng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_pca_cc_genes.R
\name{run_pca_cc_genes}
\alias{run_pca_cc_genes}
\title{Run PCA on Gene Ontology cell cycle genes}
\arguments{
\item{sce.o}{A \linkS4class{SingleCellExperiment} contains library size normalized **log-expression** matrix.}

\item{gname}{Alternative rownames of \code{sce.o}. If provided, this will be used to map genes within Gene Ontology cell cycle gene list.
If not provided, the rownames of \code{sce.o} will be used instead. Default: NULL}

\item{exprs_values}{Integer scalar or string indicating which assay of \code{sce.o} contains the **log-expression** values, which will be used to run PCA.
Default: 'logcounts'}

\item{gname.type}{The type of gene names as in \code{gname} or rownames of \code{sce.o}. It can be either 'ENSEMBL' or 'SYMBOL'. Default: 'ENSEMBL'}

\item{species}{The type of species in \code{sce.o}. It can be either 'mouse' or 'human'. If the user uses
custom \code{cycleGene.l}, this value will have no effect. Default: 'mouse'}

\item{AnnotationDb}{An AnnotationDb objects. It is used to map ENSEMBL IDs to gene SYMBOLs.
If no AnnotationDb object being given, the function will use \code{\link[org.Hs.eg.db]{org.Hs.eg.db}} or \code{\link[org.Mm.eg.db]{org.Mm.eg.db}} for human and mouse respectively.}

\item{ntop}{The number of genes with highest variance to use when calculating PCA, as in \code{\link[scater]{calculatePCA}}. Default: 500}

\item{ncomponents}{The number of component components to obtain, as in \code{\link[scater]{calculatePCA}}. Default: 20}

\item{name}{String specifying the name to be used to store the result in the \code{\link[SingleCellExperiment]{reducedDims}} of the output. Default: 'PCA'}
}
\value{
A subset \linkS4class{SingleCellExperiment} object with only GO cell cycle genes will be return.
The PCA resulting will be save in reducedDims with chosen name \code{\link[SingleCellExperiment]{reducedDims}(..., name)}.
If Seurat integration is performed, another reducedDims with name 'matched.'+\code{name} will also be included in the \linkS4class{SingleCellExperiment}.
}
\description{
Run PCA on Gene Ontology cell cycle genes abd get a new \linkS4class{SingleCellExperiment}.
User could use this function to learn new reference projection matrix.
}
\details{
The function require an output of a \linkS4class{SingleCellExperiment} object which contains the library size normalized **log-expression** matrix. The full dataset will
be subsetted to genes in the Gene Ontology cell cycle gene list (GO:0007049). The corresponding AnnotationDb object will be
 \code{\link[org.Mm.eg.db]{org.Mm.eg.db}} and \code{\link[org.Hs.eg.db]{org.Hs.eg.db}} for mouse and human respectively. If \code{runSeuratBy} is set, the data will be
 integrated to remove batch effect between samples/batches by Seurat.

User can use this function to make new reference projection matrix by getting the 'rotation' attribute in PCA results. Such as
\code{attr(reducedDim(sce.o, 'PCA'), 'rotation')[, 1:2]}. See examples for more details.
}
\examples{
data(neurosphere_example, package = "tricycle")
### Use internal NeuroRef to project and infer tricyclePosition
neurosphere_example <- estimate_cycle_position(neurosphere_example) 

### Build new reference
gocc_sce.o <- run_pca_cc_genes(neurosphere_example)
new.ref <- attr(reducedDim(gocc_sce.o, "PCA"), "rotation")[, seq_len(2)]

### Use new reference to project and infer tricyclePosition
new_sce <- estimate_cycle_position(neurosphere_example, ref.m = new.ref,
 dimred = "tricycleEmbedding2")
plot(neurosphere_example$tricyclePosition, new_sce$tricyclePosition)
}
\author{
Shijie C. Zheng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/estimate_cycle_position.R
\name{estimate_cycle_position}
\alias{estimate_cycle_position}
\title{Assign cell cycle position}
\usage{
estimate_cycle_position(
  x,
  exprs_values = "logcounts",
  dimred = "tricycleEmbedding",
  center.pc1 = 0,
  center.pc2 = 0,
  altexp = NULL,
  ...
)
}
\arguments{
\item{x}{A numeric matrix of **log-expression** values where rows are features and columns are cells.
Alternatively, a \linkS4class{SummarizedExperiment} or \linkS4class{SingleCellExperiment} containing such a matrix.}

\item{exprs_values}{Integer scalar or string indicating which assay of \code{x} contains the **log-expression** values, which will be used for projection.
If the projection already exists, you can ignore this value. Default: 'logcounts'}

\item{dimred}{The name of reducedDims in  \linkS4class{SingleCellExperiment} (\code{\link[SingleCellExperiment]{reducedDims}}). If the \code{dimred} already exists,
it will be used to assign cell cycle position. If \code{dimred} does not exist, the projection will be calculated first by \code{\link{project_cycle_space}}
and stored with name \code{dimred} in {x}. Default: 'tricycleEmbedding'}

\item{center.pc1}{The center of PC1 when defining the angle. Default: 0}

\item{center.pc2}{The center of PC2 when defining the angle. Default: 0}

\item{altexp}{String or integer scalar specifying an alternative experiment containing the **log-expression** data, which will be used for projection.
If the projection is already calculated and stored in the \linkS4class{SingleCellExperiment} as a dimred, leave this value to default NULL.}

\item{...}{Arguments to be used by \code{\link{project_cycle_space}}. If \code{x} is a \linkS4class{SingleCellExperiment}, and the projection is
already in the reducedDim with name \code{dimred}. The \code{dimred} will be directly used to assign cell cycle position withou new projection.}
}
\value{
If the input is a numeric matrix, the cell cycle position - a numeric vector bound between \eqn{0 \sim 2 \pi} with the same length as the number of input coumlum will be returned.

If the input is \linkS4class{SummarizedExperiment}, the original \linkS4class{SummarizedExperiment} with cell cycle position stored in colData with name 'tricyclePosition' will be returned.

If the input is \linkS4class{SingleCellExperiment}, the original \linkS4class{SingleCellExperiment} with cell cycle position stored in colData with name 'tricyclePosition' will be returned
and the projection will be stored in \code{\link[SingleCellExperiment]{reducedDims}(..., dimred)} if it does not exist before.
}
\description{
Assign cell cycle position by the angle formed by PC1 and PC2 in the cell cycle space.
If the cell cycle projection does not exist, the function will project the data.
}
\details{
The function will use assign the cell cycle position by the angle formed by the PC1 and PC2 of cell cycle projections.
If the input is a numeric matrix or a \linkS4class{SummarizedExperiment}, the projection will be calculated with the input **log-expression** values.
For \linkS4class{SingleCellExperiment}, the projection will also be calculated if the designated \code{dimred} does not exist.
Ohterwise, the \code{dimred} will directly be used to assign cell cycle position.
Therefore, this function is a wrapper function if the input is a \linkS4class{SingleCellExperiment}.
Refer to \code{\link{project_cycle_space}} to all arguments during the projection.

The estimated cell cycle position is bound between 0 and 2pi.
Note that we strive to get high resolution of cell cycle state, and we think
the continuous position is more appropriate when describing the cell cycle.
However, to help users understand the position variable, we also note that 
users can approximately relate 0.5pi to be the start of S stage, pi to be the
start of G2M stage, 1.5pi to be the middle of M stage, and 1.75pi-0.25pi to 
be G1/G0 stage.
}
\examples{
data(neurosphere_example, package = "tricycle")
neurosphere_example <- estimate_cycle_position(neurosphere_example)
reducedDimNames(neurosphere_example)
plot(reducedDim(neurosphere_example, "tricycleEmbedding"))
plot(neurosphere_example$tricyclePosition,
 reducedDim(neurosphere_example, "tricycleEmbedding")[, 1])
plot(neurosphere_example$tricyclePosition,
 reducedDim(neurosphere_example, "tricycleEmbedding")[, 2])
}
\references{
Zheng SC, et al.
\emph{Universal prediction of cell cycle position using transfer learning.}
Genome Biology (2022) 23: 41
doi:\href{https://doi.org/10.1186/s13059-021-02581-y}{10.1186/s13059-021-02581-y}.
}
\seealso{
\code{\link{project_cycle_space}}, for projecting new data with a pre-learned reference
}
\author{
Shijie C. Zheng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RevelioGeneList-data.R
\docType{data}
\name{RevelioGeneList}
\alias{RevelioGeneList}
\title{5 stage cell cycle gene marker list from Revelio}
\format{
An list of 5 string vector. The names of the elements are the names of cell cycle stages with the order of:
G1S, S, G2, G2M, MG1.
}
\usage{
data(RevelioGeneList)
}
\description{
This 5 stage cell cycle gene marker list is directly from \code{Revelio} package.
Within the list, 5 vectors corresponds to highly expressed genes at the cell cycle stage.
The genes are given as the human gene SYMBOLS.
This gene list is originally from the Whitfield et al.(2020).
}
\examples{
data(RevelioGeneList)
}
\references{
Whitfield ML, et al.
\emph{Identification of Genes Periodically Expressed in the Human Cell Cycle and Their Expression in Tumors.}
Molecular Biology of the Cell (2002) 13: 1977-2000
doi:\href{https://doi.org/10.1091/mbc.02-02-0030}{10.1091/mbc.02-02-0030}.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/neurosphere_example-data.R
\docType{data}
\name{neurosphere_example}
\alias{neurosphere_example}
\title{Example SingleCellExperiment dataset}
\format{
A \linkS4class{SingleCellExperiment} object of 1500 genes and 400 cells.
}
\usage{
data(neurosphere_example)
}
\description{
This a subset of mouse Neurosphere data. 200 cells from sample AX1 and AX2 were randonly
sampled from the full data. All genes in the GO cell cycle gene list and \code{\link{RevelioGeneList}}
 as well as other random 573 genes were included.
}
\examples{
data(neurosphere_example)
}
\keyword{datasets}
